from config import plant_tribes_scaffold_dir, plant_tribes_default_scaffold, hmmsearch_path, cap3_path, transdecoder_path, gt_path, trimal_path, cd_hit_dir_path, diamond_path, hmmpress_path, hmmbuild_path, clustalo_path
from lib.common.evolution.orthotools2 import read_species_info, OrthoGroups
from lib.common.fileIO import read_list_file, tsv_file_dict_parse
from lib.common.genome.seq_base import BioSeq, read_fasta_by_faidx, reverse_complement, read_fasta
from lib.common.os import cmd_run, mkdir, multiprocess_running, rmdir, move_file, get_file_name, get_file_dir
from lib.xuyuxing.base.base_function import merge_file
from lib.xuyuxing.base.common_command import merge_dict
from retry import retry
from scipy import stats
import numpy as np
import os
import re
import pickle

# build scafflod dir


def run_clustalo(in_file, aln_file, cpu_num=1):
    in_file = os.path.abspath(in_file)
    aln_file = os.path.abspath(aln_file)
    work_dir = get_file_dir(aln_file)

    if not(os.path.exists(aln_file) and os.path.getsize(aln_file) != 0):
        cmd_string = clustalo_path + " -i %s -o %s -t Protein --force --auto --threads=%d" % (
            in_file, aln_file, cpu_num)
        cmd_run(cmd_string, cwd=work_dir, silence=True)


def run_hmmbuild(in_file, hmm_file):
    in_file = os.path.abspath(in_file)
    hmm_file = os.path.abspath(hmm_file)
    work_dir = get_file_dir(hmm_file)

    if not(os.path.exists(hmm_file) and os.path.getsize(hmm_file) != 0):
        cmd_string = hmmbuild_path + " %s %s" % (hmm_file, in_file)
        cmd_run(cmd_string, cwd=work_dir, silence=True)


def build_scaffold_orthogroups(orthogroups, species_info_dict, output_dir, num_threads):
    output_dir = os.path.abspath(output_dir)

    mkdir(output_dir, True)

    orthogroups.scaffold_dir = output_dir

    orthogroups.write_OG_tsv_file(output_dir+"/orthogroups.tsv")

    orthogroups.OG_tsv_file = output_dir+"/orthogroups.tsv"

    sub_dir_dict = {
        'alns': output_dir + "/alns",
        'db': output_dir + "/db",
        'fasta': output_dir + "/fasta",
        'hmms': output_dir + "/hmms",
    }

    orthogroups.scaffold_sub_dir = sub_dir_dict

    for i in sub_dir_dict:
        mkdir(sub_dir_dict[i], True)

    sorted_og_list = sorted(orthogroups.OG_id_list, key=lambda x: len(
        orthogroups.OG_dict[x].gene_list), reverse=True)

    # write fasta
    input_aa_fasta_dict = {}
    for sp_id in species_info_dict:
        input_aa_fasta_dict[sp_id] = read_fasta_by_faidx(
            species_info_dict[sp_id].pt_file)

    input_cds_fasta_dict = {}
    for sp_id in species_info_dict:
        if species_info_dict[sp_id].cds_file:
            input_cds_fasta_dict[sp_id] = read_fasta_by_faidx(
                species_info_dict[sp_id].cds_file)

    seq_dir = sub_dir_dict['fasta']
    mkdir(seq_dir, True)

    for raw_og_id in sorted_og_list:
        cds_file = seq_dir + "/%s.fna" % raw_og_id
        aa_file = seq_dir + "/%s.faa" % raw_og_id

        og = orthogroups.OG_dict[raw_og_id]
        og.aa_seq_file = aa_file
        og.cds_seq_file = cds_file

        if not(os.path.exists(aa_file) and os.path.getsize(aa_file) != 0):

            cds_f = open(cds_file, 'w')
            aa_f = open(aa_file, 'w')

            og = orthogroups.OG_dict[raw_og_id]
            for sp_id in og.gene_dict:
                gene_list = og.gene_dict[sp_id]
                for gene in gene_list:
                    if sp_id in input_cds_fasta_dict:
                        cds_f.write(">%s\n%s\n" %
                                    (gene.id, input_cds_fasta_dict[sp_id][gene.id].seq))
                    aa_f.write(">%s\n%s\n" %
                               (gene.id, input_aa_fasta_dict[sp_id][gene.id].seq))

            cds_f.close()
            aa_f.close()

    # get aln
    aln_dir = sub_dir_dict['alns']
    mkdir(aln_dir, True)

    args_list = []
    for raw_og_id in sorted_og_list:
        aa_file = seq_dir + "/%s.faa" % raw_og_id
        aln_file = aln_dir + "/%s.aln" % raw_og_id

        og = orthogroups.OG_dict[raw_og_id]
        og.aa_aln_file = aln_file

        args_list.append((aa_file, aln_file))

    multiprocess_running(run_clustalo, args_list, num_threads, silence=True)

    # get hmm
    hmm_dir = sub_dir_dict['hmms']
    mkdir(hmm_dir, True)

    args_list = []
    for raw_og_id in sorted_og_list:
        aln_file = aln_dir + "/%s.aln" % raw_og_id
        hmm_file = hmm_dir + "/%s.hmm" % raw_og_id

        og = orthogroups.OG_dict[raw_og_id]
        og.hmm_file = hmm_file

        args_list.append((aln_file, hmm_file))

    multiprocess_running(run_hmmbuild, args_list, num_threads, silence=True)

    # get db
    # hmm
    hmm_file_list = [hmm_dir + "/%s.hmm" % raw_og_id
                     for raw_og_id in sorted_og_list if os.path.exists(hmm_dir + "/%s.hmm" % raw_og_id)]

    hmm_db_dir = sub_dir_dict['db'] + "/hmm"
    mkdir(hmm_db_dir, True)

    hmm_db_file = hmm_db_dir + "/hmm_db.hmm"
    merge_file(hmm_file_list, hmm_db_file)

    cmd_string = hmmpress_path + " " + hmm_db_file
    cmd_run(cmd_string, cwd=hmm_db_dir, silence=True)

    orthogroups.hmm_db = hmm_db_file

    # diamond
    aa_file_list = [seq_dir + "/%s.faa" % raw_og_id
                    for raw_og_id in sorted_og_list if os.path.exists(seq_dir + "/%s.faa" % raw_og_id)]

    diamond_db_dir = sub_dir_dict['db'] + "/diamond"
    mkdir(diamond_db_dir, True)

    diamond_db_file = diamond_db_dir + "/diamond_db.faa"
    merge_file(aa_file_list, diamond_db_file)

    cmd_string = "%s makedb --in %s --db %s" % (
        diamond_path, diamond_db_file, diamond_db_file)
    cmd_run(cmd_string, cwd=diamond_db_dir, silence=True)

    orthogroups.hmm_db = diamond_db_file+".dmnd"

    orthogroups.pickle_file = orthogroups.scaffold_dir + "/OG_pickle.py"

    OUT = open(orthogroups.pickle_file, 'wb')
    pickle.dump(orthogroups, OUT)
    OUT.close()

    return orthogroups


def build_scaffold_main(args):
    # read orthogroups
    orthogroups = OrthoGroups(OG_tsv_file=args.orthogroups_file)

    og_list = []
    for og_id in orthogroups.OG_id_list:
        og = orthogroups.OG_dict[og_id]
        g_num = len(og.gene_list)
        s_num = len([i for i in og.species_stat if og.species_stat[i] != 0])
        if g_num >= args.min_gene_number and s_num >= args.min_species_number:
            og_list.append(og)

    orthogroups = OrthoGroups(from_OG_list=og_list)

    # read file of files
    species_info_dict = read_species_info(args.species_info_file)

    # build scaffold dir
    build_scaffold_orthogroups(orthogroups, species_info_dict,
                               args.scaffold_dir, args.num_threads)


# seq_classify
def read_hmm_out_get_subject_id_list(hmm_out_file):
    subject_id_list = []
    with open(hmm_out_file, 'r') as f:
        for each_line in f:
            if not re.match(r'^#', each_line):
                s_id = each_line.split()[0]
                subject_id_list.append(s_id)
    return list(set(subject_id_list))


def read_hmm_out_get_subject_id_score_dict(hmm_out_file):
    score_dict = {}
    num = 0
    with open(hmm_out_file, 'r') as f:
        for each_line in f:
            if not re.match(r'^#', each_line):
                s_id = each_line.split()[0]
                score = float(each_line.split()[5])
                if s_id in score_dict:
                    if score > score_dict[s_id]:
                        score_dict[s_id] = score
                else:
                    score_dict[s_id] = score
            num += 1
            if num % 1000000 == 0:
                print(num)
    return score_dict


def read_hmm_out_get_query_id_score_dict(hmm_out_file):
    score_dict = {}
    num = 0
    with open(hmm_out_file, 'r') as f:
        for each_line in f:
            if not re.match(r'^#', each_line):
                q_id = each_line.split()[2]
                score = float(each_line.split()[5])
                if q_id in score_dict:
                    if score > score_dict[q_id]:
                        score_dict[q_id] = score
                else:
                    score_dict[q_id] = score
            num += 1
            if num % 1000000 == 0:
                print(num)
    return score_dict


def seq_in_ortho_stat(protein_seq, scaffold_hmm_file, scaffold_seq_fasta, scaffold_msa_file, work_dir, num_threads=1):
    mkdir(work_dir, True)

    input_fasta = work_dir + "/input.faa"

    with open(input_fasta, 'w') as f:
        f.write(">%s\n%s" % ("input_now", protein_seq.seq))

    # get stats
    # hmm score
    hmm_out_file = work_dir + "/pep.hmm.tbl"
    hmm_log_file = work_dir + "/pep.hmm.log"

    cmd_string = '%s -E 1e-20 --cpu %d --noali --tblout %s -o %s %s %s >/dev/null' % (
        hmmsearch_path, num_threads, hmm_out_file, hmm_log_file, scaffold_hmm_file, input_fasta)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    backbone_hmm_out_file = work_dir + "/backbone.hmm.tbl"
    backbone_hmm_log_file = work_dir + "/backbone.hmm.log"

    cmd_string = '%s -E 1e-20 --cpu %d --noali --tblout %s -o %s %s %s >/dev/null' % (
        hmmsearch_path, num_threads, backbone_hmm_out_file, backbone_hmm_log_file, scaffold_hmm_file, scaffold_seq_fasta)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    backbone_score_dict = read_hmm_out_get_subject_id_score_dict(
        backbone_hmm_out_file)
    pep_score_dict = read_hmm_out_get_subject_id_score_dict(
        hmm_out_file)

    backbone_score = [backbone_score_dict[i] for i in backbone_score_dict]

    avg_backbone_score = sum(backbone_score)/len(backbone_score)
    std_backbone_score = np.std(np.array(backbone_score))

    # aln
    pep_aln_file = work_dir + "/pep.aln"
    cmd_string = "mafft --add %s %s > %s" % (
        input_fasta, scaffold_msa_file, pep_aln_file)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    pep_trim_aln_file = work_dir + "/pep.trim.aln"
    cmd_string = "%s -in %s -out %s -gt 0.1 >/dev/null" % (
        trimal_path, pep_aln_file, pep_trim_aln_file)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    backbone_seq_id_list = list(read_fasta_by_faidx(scaffold_seq_fasta).keys())
    input_seq_id_list = list(read_fasta_by_faidx(input_fasta).keys())

    aln_seq_dict = read_fasta_by_faidx(pep_trim_aln_file)

    if not (aln_seq_dict[list(aln_seq_dict.keys())[0]].len() == 0):
        stat_dict = {}
        for seq_id in aln_seq_dict:
            aln_seq = aln_seq_dict[seq_id].seq
            aln_cov = len(aln_seq.replace("-", ""))/len(aln_seq)
            stat_dict[seq_id] = (len(aln_seq.replace("-", "")),
                                 len(aln_seq), aln_cov)

        backbone_len = [stat_dict[i][0]
                        for i in stat_dict if i in backbone_seq_id_list]
        backbone_cov = [stat_dict[i][2]
                        for i in stat_dict if i in backbone_seq_id_list]

        if len(backbone_len):
            avg_backbone_len = sum(backbone_len)/len(backbone_len)
        else:
            avg_backbone_len = 0
        std_backbone_len = np.std(np.array(backbone_len))

        avg_backbone_cov = sum(backbone_cov)/len(backbone_cov)
        std_backbone_cov = np.std(np.array(backbone_cov))

    else:
        avg_backbone_len = 0.0
        std_backbone_len = 0.0
        avg_backbone_cov = 0.0
        std_backbone_cov = 0.0
        stat_dict = {i: (0, 0, 0.0) for i in input_seq_id_list}

    i = "input_now"
    if i in pep_score_dict:
        score = pep_score_dict[i]
    else:
        score = 0.0

    # col_list = ["seq_id", "cov", "avg_cov", "sd_cov", "len", "avg_len", "sd_len", "score", "avg_score", "sd_score"]

    rmdir(work_dir)

    return protein_seq.seqname, stat_dict[i][2], avg_backbone_cov, std_backbone_cov, stat_dict[i][0], avg_backbone_len, std_backbone_len, score, avg_backbone_score, std_backbone_score


def make_seqs_stat_file(input_fasta, scaffold_hmm_file=None, scaffold_seq_fasta=None, scaffold_msa_file=None, work_dir=None, num_threads=1):
    mkdir(work_dir, True)

    # get stats
    # hmm score
    if scaffold_hmm_file:
        hmm_out_file = work_dir + "/pep.hmm.tbl"
        hmm_log_file = work_dir + "/pep.hmm.log"

        cmd_string = '%s -E 1e-20 --cpu %d --noali --tblout %s -o %s %s %s >/dev/null' % (
            hmmsearch_path, num_threads, hmm_out_file, hmm_log_file, scaffold_hmm_file, input_fasta)
        cmd_run(cmd_string, silence=True, cwd=work_dir)

        backbone_hmm_out_file = work_dir + "/backbone.hmm.tbl"
        backbone_hmm_log_file = work_dir + "/backbone.hmm.log"

        cmd_string = '%s -E 1e-20 --cpu %d --noali --tblout %s -o %s %s %s >/dev/null' % (
            hmmsearch_path, num_threads, backbone_hmm_out_file, backbone_hmm_log_file, scaffold_hmm_file, scaffold_seq_fasta)
        cmd_run(cmd_string, silence=True, cwd=work_dir)

        backbone_score_dict = read_hmm_out_get_subject_id_score_dict(
            backbone_hmm_out_file)
        pep_score_dict = read_hmm_out_get_subject_id_score_dict(
            hmm_out_file)

        backbone_score = [backbone_score_dict[i] for i in backbone_score_dict]

        avg_backbone_score = sum(backbone_score)/len(backbone_score)
        std_backbone_score = np.std(np.array(backbone_score))
    else:
        pep_score_dict = {}
        avg_backbone_score = 0.0
        std_backbone_score = 0.0

    # aln
    pep_aln_file = work_dir + "/pep.aln"
    cmd_string = "mafft --add %s %s > %s" % (
        input_fasta, scaffold_msa_file, pep_aln_file)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    pep_trim_aln_file = work_dir + "/pep.trim.aln"
    cmd_string = "%s -in %s -out %s -gt 0.1 >/dev/null" % (
        trimal_path, pep_aln_file, pep_trim_aln_file)
    cmd_run(cmd_string, silence=True, cwd=work_dir)

    backbone_seq_id_list = list(read_fasta_by_faidx(scaffold_seq_fasta).keys())
    input_seq_id_list = list(read_fasta_by_faidx(input_fasta).keys())

    aln_seq_dict = read_fasta_by_faidx(pep_trim_aln_file)

    if not (aln_seq_dict[list(aln_seq_dict.keys())[0]].len() == 0):
        stat_dict = {}
        for seq_id in aln_seq_dict:
            aln_seq = aln_seq_dict[seq_id].seq
            aln_cov = len(aln_seq.replace("-", ""))/len(aln_seq)
            stat_dict[seq_id] = (len(aln_seq.replace("-", "")),
                                 len(aln_seq), aln_cov)

        backbone_len = [stat_dict[i][0]
                        for i in stat_dict if i in backbone_seq_id_list]
        backbone_cov = [stat_dict[i][2]
                        for i in stat_dict if i in backbone_seq_id_list]

        if len(backbone_len):
            avg_backbone_len = sum(backbone_len)/len(backbone_len)
            std_backbone_len = np.std(np.array(backbone_len))

            avg_backbone_cov = sum(backbone_cov)/len(backbone_cov)
            std_backbone_cov = np.std(np.array(backbone_cov))
        else:
            avg_backbone_len = 0.0
            std_backbone_len = 0.0
            avg_backbone_cov = 0.0
            std_backbone_cov = 0.0
            stat_dict = {i: (0, 0, 0.0) for i in input_seq_id_list}

    else:
        avg_backbone_len = 0.0
        std_backbone_len = 0.0
        avg_backbone_cov = 0.0
        std_backbone_cov = 0.0
        stat_dict = {i: (0, 0, 0.0) for i in input_seq_id_list}

    with open(work_dir+"/stats.txt", 'w') as f:
        col_list = ["seq_id", "cov", "avg_cov",
                    "sd_cov", "len", "avg_len", "sd_len", "score", "avg_score", "sd_score"]
        f.write("\t".join(col_list)+"\n")

        for i in input_seq_id_list:
            if i in pep_score_dict:
                score = pep_score_dict[i]
            else:
                score = 0.0

            write_list = [i, "%.2f" % stat_dict[i][2], "%.2f" % avg_backbone_cov, "%.2f" % std_backbone_cov,
                          "%.2f" % stat_dict[i][0], "%.2f" % avg_backbone_len, "%.2f" % std_backbone_len,
                          "%.2f" % score, "%.2f" % avg_backbone_score, "%.2f" % std_backbone_score, ]
            f.write("\t".join(write_list)+"\n")


def get_pass_seq_id(stats_file, p_value=0.05, only_cov=False):
    pass_seq_list = []

    if only_cov:
        judge_list = ['cov']
    else:
        judge_list = ['cov', 'score']

    if os.path.exists(stats_file) and os.path.getsize(stats_file) != 0:

        stats_info_dict = tsv_file_dict_parse(stats_file, key_col='seq_id')

        for seq_id in stats_info_dict:
            seq_info = stats_info_dict[seq_id]
            pass_flag = True
            for i in judge_list:
                seq_value = float(seq_info[i])
                avg_value = float(seq_info['avg_'+i])
                sd_value = float(seq_info['sd_'+i])

                if avg_value > 0.0:
                    distn = stats.norm(avg_value, sd_value)
                    if seq_value < distn.ppf(p_value):
                        pass_flag = False
                else:
                    pass_flag = False

            if pass_flag:
                pass_seq_list.append(seq_id)

    return pass_seq_list


def if_seq_in_ortho(protein_seq, scaffold_hmm_file, scaffold_seq_fasta, scaffold_msa_file, work_dir, clean_up=True, num_threads=1, only_cov=False):
    scaffold_hmm_file = os.path.abspath(scaffold_hmm_file)
    scaffold_seq_fasta = os.path.abspath(scaffold_seq_fasta)
    scaffold_msa_file = os.path.abspath(scaffold_msa_file)
    work_dir = os.path.abspath(work_dir)

    mkdir(work_dir, True)

    input_fasta = work_dir + "/input.faa"

    with open(input_fasta, 'w') as f:
        f.write(">%s\n%s" % ("input_now", protein_seq.seq))

    if only_cov:
        make_seqs_stat_file(input_fasta, None,
                        scaffold_seq_fasta, scaffold_msa_file, work_dir, num_threads=num_threads)
    else:
        make_seqs_stat_file(input_fasta, scaffold_hmm_file,
                        scaffold_seq_fasta, scaffold_msa_file, work_dir, num_threads=num_threads)

    stat_file = work_dir + "/stats.txt"

    pass_id_list = get_pass_seq_id(stat_file, p_value=0.05, only_cov=only_cov)

    if clean_up:
        rmdir(work_dir)

    if len(pass_id_list) > 0:
        return True
    else:
        return False


def get_best_ortho(seqname, hmm_results, diamond_out, scaffold_dir):
    hmm_score_dict = read_hmm_out_get_query_id_score_dict(hmm_results)

    if len(hmm_score_dict) > 0:
        hmm_top_og_id = sorted(
            hmm_score_dict, key=lambda x: hmm_score_dict[x], reverse=True)[0]
    else:
        hmm_top_og_id = None

    diamond_top_gene_id = None
    if diamond_out:
        with open(diamond_out, 'r') as f:
            all_info = f.read()
            if len(all_info) > 0:
                diamond_top_gene_id = all_info.split("\t")[1]

    if diamond_top_gene_id:
        og_tsv = scaffold_dir + "/orthogroups.tsv"
        with open(og_tsv, 'r') as f:
            for each_line in f:
                if diamond_top_gene_id in each_line:
                    diamond_top_og_id = each_line.split("\t")[0]
    else:
        diamond_top_og_id = None

    if hmm_top_og_id:
        top_og_id = hmm_top_og_id
    elif hmm_top_og_id is None:
        top_og_id = diamond_top_og_id

    return top_og_id


@retry(tries=5)
def seq_classify(protein_seq, scaffold_dir, work_dir, strict=True, clean_up=True):
    mkdir(work_dir, True)
    input_fasta = work_dir + "/input.faa"

    with open(input_fasta, 'w') as f:
        f.write(">%s\n%s" % (protein_seq.seqname, protein_seq.seq))

    # get best ortho
    hmm_db = scaffold_dir + "/db/hmm/hmm_db.hmm"

    hmm_results = work_dir + "/hmm.tbl"
    hmm_log = work_dir + "/hmm.log"

    cmd_string = '%s -E 1e-20 --cpu 1 --noali --tblout %s -o %s %s %s >/dev/null' % (
        hmmsearch_path, hmm_results, hmm_log, hmm_db, input_fasta)
    cmd_run(cmd_string, cwd=work_dir, silence=True)

    if strict:
        diamond_db = scaffold_dir + "/db/diamond/diamond_db.faa.dmnd"

        diamond_out = work_dir + "/diamond.bls"

        cmd_string = "diamond blastp -q %s -k 50 -e 1e-5 -d %s -o %s -f 6 -p 1" % (
            input_fasta, diamond_db, diamond_out)

        cmd_run(cmd_string, cwd=work_dir, silence=True)
    else:
        diamond_out = None

    best_ortho = get_best_ortho(
        protein_seq.seqname, hmm_results, diamond_out, scaffold_dir)

    if strict:
        pass_flag = False
        if best_ortho:
            scaffold_hmm_file = "%s/hmms/%s.hmm" % (scaffold_dir, best_ortho)
            scaffold_seq_fasta = "%s/fasta/%s.faa" % (scaffold_dir, best_ortho)
            scaffold_msa_file = "%s/alns/%s.aln" % (scaffold_dir, best_ortho)
            check_dir = work_dir + "/check_dir"

            pass_flag = if_seq_in_ortho(protein_seq, scaffold_hmm_file,
                                        scaffold_seq_fasta, scaffold_msa_file, check_dir, clean_up, 1)
    else:
        pass_flag = True

    if clean_up:
        rmdir(work_dir)

    if best_ortho and pass_flag:
        return best_ortho
    else:
        return None


def seq_classify_main(args):
    args.scaffold_dir = os.path.abspath(args.scaffold_dir)
    args.input_fasta = os.path.abspath(args.input_fasta)
    args.output_file = os.path.abspath(args.output_file)

    if not args.work_dir:
        args.work_dir = get_file_dir(args.input_fasta) + "/tmp"
    else:
        args.work_dir = os.path.abspath(args.work_dir)

    mkdir(args.work_dir, True)

    seq_dict = read_fasta_by_faidx(args.input_fasta)

    args_list = []
    args_id_list = []
    for seq_id in seq_dict:
        work_dir = args.work_dir + "/" + seq_id
        seq_record = seq_dict[seq_id]
        seq_record = BioSeq(seq_record.seq, seq_record.seqname)
        args_list.append((seq_record, args.scaffold_dir, work_dir, args.strict, True))
        args_id_list.append(seq_id)

    mlt_out = multiprocess_running(
        seq_classify, args_list, args.thread_number, None, False, args_id_list)

    with open(args.output_file, 'w') as f:
        for seq_id in mlt_out:
            best_ortho = mlt_out[seq_id]['output']
            if best_ortho:
                f.write("%s\t%s\n" % (seq_id, best_ortho))

    rmdir(args.work_dir)


if __name__ == '__main__':
    from lib.common.evolution.orthotools2 import read_species_info, OrthoGroups

    species_info_file = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/Oro_ref.xlsx"
    orthogroups_file = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/tree/orthologous_OGs.tsv"
    num_threads = 56

    # read orthogroups
    orthogroups = OrthoGroups(OG_tsv_file=orthogroups_file)

    og_list = []
    for i in orthogroups.OG_id_list[18000:19000]:
        og_list.append(orthogroups.OG_dict[i])

    orthogroups = OrthoGroups(from_OG_list=og_list)

    # read file of files
    species_info_dict = read_species_info(species_info_file)

    # build scaffold dir
    scaffold_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/gene_loss/gene_loss/Ocu/test2"
    scaf_OGs = build_scaffold_orthogroups(orthogroups, species_info_dict,
                                          scaffold_dir, num_threads)

    # big seq_classify
    "hmmsearch -E 1e-20 --cpu 1 --noali --tblout all.input.fasta.tbl -o all.input.fasta.log hmm_db.hmm all.input.fasta"

    import os
    import re
    from lib.common.genome.seq_base import read_fasta
    from api.xuyuxing.comp_genome.planttribes2 import if_seq_in_ortho
    from lib.common.os import multiprocess_running

    hmm_out_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/hmm/all.input.fasta.tbl'

    score_dict = {}
    num = 0
    with open(hmm_out_file, 'r') as f:
        for each_line in f:
            if not re.match(r'^#', each_line):
                q_id = each_line.split()[2]
                s_id = each_line.split()[0]
                score = float(each_line.split()[5])

                score_dict.setdefault(s_id,(q_id, score))
                
                if score > score_dict[s_id][1]:
                    score_dict[s_id] = (q_id, score)

            num += 1
            if num % 1000000 == 0:
                print(num)

    sp_seq_dict = {}
    for seq_id in score_dict:
        sp_id = seq_id.split("_")[0] + "_" +seq_id.split("_")[1]
        sp_seq_dict.setdefault(sp_id, {})
        sp_seq_dict[sp_id][seq_id] = score_dict[seq_id]

    seqclassify_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify"

    for sp_id in sp_seq_dict:
        output_file = seqclassify_dir + "/" + sp_id + "/output.txt"
        with open(output_file, 'w') as f:
            for seq_id in sp_seq_dict[sp_id]:
                og_id, score = sp_seq_dict[sp_id][seq_id]
                f.write("%s\t%s\n" % (seq_id, og_id))

    Tri_ver_output = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/Reh_glu/output.txt"
    Tri_ver_filter_output = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/Reh_glu/output.filter.txt"
    Tri_ver_ctm = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/Reh_glu/no.ctm.id"

    from lib.common.fileIO import read_list_file
    from lib.common.fileIO import tsv_file_dict_parse

    Tri_ver_ctm_list = set(read_list_file(Tri_ver_ctm))

    raw_dict = tsv_file_dict_parse(Tri_ver_output, fieldnames=['g_id', 'og_id'], key_col='g_id')
    with open(Tri_ver_filter_output, 'w') as f:
        for g_id in raw_dict:
            if g_id in Tri_ver_ctm_list:
                f.write("%s\t%s\n" % (g_id, raw_dict[g_id]['og_id']))




    ##

    seq_dir = '/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/1.clean_data/renamed_data'

    seq_dict = {}
    for i in os.listdir(seq_dir):
        f_name = re.findall(r'.*\.pt\.faa$', i)
        if len(f_name):
            f_name = f_name[0]
            sp_id = f_name.split(".")[0]
            print(sp_id)
            f_name = seq_dir + "/" + f_name
            seq_dict[sp_id] = read_fasta(f_name)[0]

    ##
    scaffold_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/2.scaffold/scaffold"
    work_dir = "/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/hmm/tmp"

    args_list = []
    args_id_list = []

    for seq_id in score_dict:
        og_id, score = score_dict[seq_id]
        sp_id = seq_id.split("_")[0] + "_" +seq_id.split("_")[1]

        best_ortho = score_dict[seq_id][0]

        scaffold_hmm_file = "%s/hmms/%s.hmm" % (scaffold_dir, best_ortho)
        scaffold_seq_fasta = "%s/fasta/%s.faa" % (scaffold_dir, best_ortho)
        scaffold_msa_file = "%s/alns/%s.aln" % (scaffold_dir, best_ortho)

        protein_seq = seq_dict[sp_id][seq_id]

        work_dir_tmp = work_dir + "/" + seq_id

        args_list.append((protein_seq, scaffold_hmm_file, scaffold_seq_fasta, scaffold_msa_file, work_dir_tmp, True, 1, True))
        args_id_list.append(seq_id)

    mlt_out = multiprocess_running(if_seq_in_ortho, args_list, 80, None, False, args_id_list)

    with open("/lustre/home/xuyuxing/Work/Orobanchaceae/HGT/3.init_filter/3.1.seqclassify/hmm/output.txt", 'w') as f:
        for seq_id in mlt_out:
            if mlt_out[seq_id]['output']:
                f.write("%s\t%s\n" % (seq_id, score_dict[seq_id][0]))


from pyfaidx import Fasta
from collections import OrderedDict
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.SeqFeature import SeqFeature, FeatureLocation
from BCBio import GFF
from lib.common.genome.seq_base import reverse_complement, read_fasta_by_faidx, get_seq_index_ignore_gap
from lib.common.os import rmdir, cmd_run
from lib.common.util import printer_list
from lib.common.util import logging_init
from api.xuyuxing.genome.trf import trf_dat_to_dict
from lib.common.math.set import merge_same_element_set
from lib.common.math.interval import section, merge_intervals
from itertools import combinations
from api.common.genome.blast import outfmt5_read_big, blastn_running, outfmt5_read
from api.xuyuxing.genome.genblasta import GenBlastAJob
from lib.common.fileIO import tsv_file_dict_parse
import os

"""
class abc(object):
    pass

args = abc()

args.fasta_file = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/rRNA/test/Cuscuta.genome.v1.1.fasta"
args.query_rRNA_seq = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/rRNA/test/Ath/rRNA.fa"
args.query_rRNA_bed = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/rRNA/test/Ath/rRNA.bed"
args.output_dir = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/rRNA/test/Cau_rRNA"
args.log_file = "/lustre/home/xuyuxing/Database/Cuscuta/Cau/genomev1.1/rRNA/test/log"
"""

script_dir_path = os.path.split(os.path.realpath(__file__))[0]

ath_query_rRNA_seq = script_dir_path+"/Ath_rRNA/rRNA.fa"
ath_query_rRNA_bed = script_dir_path+"/Ath_rRNA/rRNA.bed"

ath_query_rRNA_seq = os.path.abspath(ath_query_rRNA_seq)
ath_query_rRNA_bed = os.path.abspath(ath_query_rRNA_bed)


def rRNAFinder_main(args):
    args.fasta_file = os.path.abspath(args.fasta_file)

    if args.query_rRNA_seq:
        args.query_rRNA_seq = os.path.abspath(args.query_rRNA_seq)
    else:
        args.query_rRNA_seq = ath_query_rRNA_seq

    if args.query_rRNA_bed:
        args.query_rRNA_bed = os.path.abspath(args.query_rRNA_bed)
    else:
        args.query_rRNA_bed = ath_query_rRNA_bed

    args.output_dir = os.path.abspath(args.output_dir)
    if not args.log_file is None:
        args.log_file = os.path.abspath(args.log_file)

    # get rRNA unit

    # genblasta
    query_seq_dir = read_fasta_by_faidx(args.query_rRNA_seq)
    target_seq_dir = read_fasta_by_faidx(args.fasta_file)

    query_seq_list = [query_seq_dir[i] for i in query_seq_dir]
    target_seq_list = (target_seq_dir[i] for i in target_seq_dir)

    genblasta_job = GenBlastAJob(
        query_seq_list, target_seq_list, args.output_dir, blast_type='n2n')
    genblasta_job.run()

    genblasta_out_dir = {i[0]: i[1] for i in genblasta_job.parse_output(True)}

    best_genblast_hit = genblasta_out_dir[query_seq_list[0].seqname][0]

    # extract best subject hit
    best_hit_range = best_genblast_hit.rangeB

    s_file = genblasta_job.work_dir + "/best_gba_hit_flank.fa"

    s_flank_start = max(best_hit_range.start - 1000, 1)
    s_flank_end = min(best_hit_range.end + 1000,
                      target_seq_dir[best_hit_range.chr_id].seqs_length())
    s_seq = str(
        target_seq_dir[best_hit_range.chr_id].faidx[s_flank_start - 1:s_flank_end])
    s_base = s_flank_start
    if best_hit_range.strand == "-":
        s_seq = reverse_complement(s_seq)
        s_base = s_flank_end

    with open(s_file, 'w') as s:
        s.write(">%s\n%s" % ("best_gba_hit_flank", s_seq))

    # blastn
    blast_out_file = genblasta_job.work_dir + "/best_gba_hit_flank.bls"

    cmd_string = "makeblastdb -in %s -dbtype nucl" % s_file
    cmd_run(cmd_string, cwd=genblasta_job.work_dir, silence=True)
    cmd_string = "blastn -query %s -db %s -out %s -outfmt 5 -task blastn -evalue 1e-10 " % (
        args.query_rRNA_seq, s_file, blast_out_file)
    cmd_run(cmd_string, cwd=genblasta_job.work_dir, silence=True)

    blastn_output = outfmt5_read(blast_out_file)
    hsp_list = blastn_output[query_seq_list[0].seqname].hit[0].hsp

    query_str_dict = tsv_file_dict_parse(args.query_rRNA_bed, seq=",", key_col='name',
                                         fieldnames=['name', 'contig', 'start', 'end'])

    # get each rRNA gene
    rRNA_gene_file = args.output_dir + "/rRNA.gene.fa"

    with open(rRNA_gene_file, 'w') as f:
        type_seq_dict = {}
        for type_name in ['18S', '5.8S', '25S']:
            q_start = int(query_str_dict[type_name]['start'])
            q_end = int(query_str_dict[type_name]['end'])

            # get type overlap hsp
            over_hsp = [j for j in
                        [(section((q_start, q_end), (i.Hsp_query_from, i.Hsp_query_to)), i) for i in hsp_list] if
                        j[0][0]]

            if len(over_hsp) == 0:
                raise ValueError("can't find %s" % type_name)

            # get longest overlap hsp
            section_out, type_hsp = sorted(
                over_hsp, key=lambda x: x[0][1][1] - x[0][1][0], reverse=True)[0]

            deta = section_out[1]

            alig_start = get_seq_index_ignore_gap(
                deta[0], type_hsp.Hsp_qseq, type_hsp.Hsp_query_from, '-')
            alig_end = get_seq_index_ignore_gap(
                deta[1], type_hsp.Hsp_qseq, type_hsp.Hsp_query_from, '-')

            sub_type_seq = type_hsp.Hsp_hseq[alig_start -
                                             1:alig_end].replace('-', '')

            type_seq_dict[type_name] = sub_type_seq

            f.write(">%s\n%s\n" % (type_name, sub_type_seq))

    # blastn again
    gene_vs_flank_bls = genblasta_job.work_dir + "/gene_vs_flank.bls"

    cmd_string = "blastn -query %s -db %s -out %s -outfmt 5 -task blastn -evalue 1e-10 " % (
        rRNA_gene_file, s_file, gene_vs_flank_bls)
    cmd_run(cmd_string, cwd=genblasta_job.work_dir, silence=True)

    blastn_output = outfmt5_read(gene_vs_flank_bls)

    type_range = {}
    for type_name in ['18S', '5.8S', '25S']:
        top_hit = blastn_output[type_name].hit[0].hsp[0]
        type_range[type_name] = (top_hit.Hsp_hit_from, top_hit.Hsp_hit_to)

    type_range['ITS1'] = (type_range['18S'][1] + 1, type_range['5.8S'][0] - 1)
    type_range['ITS2'] = (type_range['5.8S'][1] + 1, type_range['25S'][0] - 1)

    type_range_dict = OrderedDict()
    for i in ['18S', 'ITS1', '5.8S', 'ITS2', '25S']:
        type_range_dict[i] = type_range[i]

    # output
    flank_seq = read_fasta_by_faidx(s_file)['best_gba_hit_flank']

    unit_fasta_file = args.output_dir + "/rRNA.unit.fa"
    gene_type_fasta_file = args.output_dir + "/rRNA.gene.fa"
    unit_gff_file = args.output_dir + "/rRNA.unit.gff"

    rna_unit = Seq(flank_seq.sub(
        type_range_dict['18S'][0], type_range_dict['25S'][1], "+", False))
    rec = SeqRecord(rna_unit, "rRNA_unit", description='')

    with open(unit_fasta_file, 'w') as f:
        f.write(rec.format('fasta'))

    type_noflank_range_dict = OrderedDict()
    flank_start = type_range_dict['18S'][0]
    with open(gene_type_fasta_file, 'w') as f:
        for i in ['18S', 'ITS1', '5.8S', 'ITS2', '25S']:
            type_noflank_range_dict[i] = (
                type_range_dict[i][0] - flank_start + 1, type_range_dict[i][1] - flank_start + 1)

            type_seq = Seq(flank_seq.sub(
                type_range_dict[i][0], type_range_dict[i][1], "+", False))
            type_rec = SeqRecord(type_seq, i, description='')
            f.write(type_rec.format('fasta'))

    rec.features = []
    qualifiers = {
        "source": "rRNAFinder",
        "ID": 'rRNA'
    }
    top_feature = SeqFeature(FeatureLocation(type_noflank_range_dict['18S'][0] - 1,
                                             type_noflank_range_dict['25S'][1], strand=1),
                             type="rRNA", qualifiers=qualifiers)
    top_feature.sub_features = []
    for i in ['18S', 'ITS1', '5.8S', 'ITS2', '25S']:

        sub_qualifiers = {
            "source": "rRNAFinder",
            "ID": i
        }
        if i == 'ITS1' or i == 'ITS2':
            type_tmp = 'intron'
        else:
            type_tmp = 'rRNA'

        sub_feature = SeqFeature(FeatureLocation(type_noflank_range_dict[i][0] - 1,
                                                 type_noflank_range_dict[i][1], strand=1),
                                 type=type_tmp, qualifiers=sub_qualifiers)

        top_feature.sub_features.append(sub_feature)

    rec.features.append(top_feature)

    with open(unit_gff_file, "w") as f:
        GFF.write([rec], f)

    rmdir(genblasta_job.work_dir)



import gzip
import os
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, Float, ForeignKey, create_engine
from sqlalchemy.orm import relationship, sessionmaker

Base = declarative_base()


class BlastQuery(Base):
    __tablename__ = 'Query'

    num = Column(Integer, primary_key=True)
    qID = Column(String)
    qDef = Column(String)
    qLen = Column(Integer)

    def __repr__(self):
        return "<BlastQuery(ID='%d', query_name='%s')>" % (
            self.num, self.qDef)


class BlastHit(Base):
    __tablename__ = 'Hit'

    hit_num = Column(Integer, primary_key=True)
    query_num = Column(Integer, ForeignKey('Query.num'))
    hit_def = Column(String)
    hit_accession = Column(String)
    hit_len = Column(Integer)

    query = relationship("BlastQuery", back_populates="hit")

    def __repr__(self):
        return "<BlastHit(hit_num='%d', hit_name='%s')>" % (
            self.hit_num, self.hit_def)


BlastQuery.hit = relationship(
    "BlastHit", order_by=BlastHit.hit_num, back_populates="query")


class BlastHsp(Base):
    __tablename__ = 'Hsp'

    hsp_num = Column(Integer, primary_key=True)
    hit_num = Column(Integer, ForeignKey('Hit.hit_num'))
    query_num = Column(Integer, ForeignKey('Query.num'))
    hsp_bit_score = Column(Float)
    hsp_score = Column(Integer)
    hsp_evalue = Column(Float)
    hsp_query_from = Column(Integer)
    hsp_query_to = Column(Integer)
    hsp_hit_from = Column(Integer)
    hsp_hit_to = Column(Integer)
    hsp_query_frame = Column(Integer)
    hsp_hit_frame = Column(Integer)
    hsp_identity = Column(Float)
    hsp_positive = Column(Float)
    hsp_gaps = Column(Integer)
    hsp_align_len = Column(Integer)
    hsp_qseq = Column(String)
    hsp_hseq = Column(String)
    hsp_midline = Column(String)
    hsp_mismatch = Column(Integer)

    query = relationship("BlastQuery", back_populates="hsp")
    hit = relationship("BlastHit", back_populates="hsp")

    def __repr__(self):
        return "<BlastHsp(hsp_num='%d', hsp_bit_score='%f')>" % (
            self.hsp_num, self.hsp_bit_score)


BlastHit.hsp = relationship(
    "BlastHsp", order_by=BlastHsp.hsp_num, back_populates="hit")

BlastQuery.hsp = relationship(
    "BlastHsp", order_by=BlastHsp.hsp_num, back_populates="query")


def outfmt6_read_big(input_file, gzip_flag=False, query_num_start=0, hit_num_start=0, hsp_num_start=0):
    fieldname = ["q_id", "s_id", "identity", "aln_len", "mis", "gap", "q_start", "q_end", "s_start", "s_end", "e_value",
                 "score"]

    if gzip_flag:
        f = gzip.open(input_file, 'rt')
    else:
        f = open(input_file, 'r')

    query_num = query_num_start
    hit_num = hit_num_start
    hsp_num = hsp_num_start
    query_tmp = None
    for each_line in f:
        each_line = each_line.strip()
        info = each_line.split("\t")

        if query_tmp is None:
            new_query_flag = 1
            new_hit_flag = 1
        else:
            last_query_def = query_tmp.qDef
            last_hit_def = query_tmp.hit[-1].hit_def
            if last_query_def != info[0]:
                new_query_flag = 1
                new_hit_flag = 1
            else:
                new_query_flag = 0
                if last_hit_def != info[1]:
                    new_hit_flag = 1
                else:
                    new_hit_flag = 0

        if new_query_flag:
            if query_tmp is not None:
                # print(query_tmp)
                yield query_tmp

            query_num = query_num + 1
            query_tmp = BlastQuery(num=query_num, qID=info[0], qDef=info[0])
            # hit_num = 0

        if new_hit_flag:
            hit_num = hit_num + 1
            hit_tmp = BlastHit(hit_num=hit_num, query_num=query_num, hit_def=info[1], hit_accession=info[1])
            query_tmp.hit.append(hit_tmp)
            # hsp_num = 0

        hsp_num = hsp_num + 1
        hsp_tmp = BlastHsp(query_num=query_num, hsp_num=hsp_num, hit_num=hit_num, hsp_bit_score=float(info[11]),
                           hsp_evalue=float(info[10]), hsp_query_frame=int(info[6]), hsp_query_to=int(info[7]),
                           hsp_hit_from=int(info[8]), hsp_hit_to=int(info[9]), hsp_identity=float(info[2]),
                           hsp_gaps=int(info[5]), hsp_align_len=int(info[3]), hsp_mismatch=int(info[4]))

        # hsp_tmp.Hsp_mismatch = int(info[4])
        query_tmp.hit[-1].hsp.append(hsp_tmp)

    if query_tmp is not None:
        # print(query_tmp)
        yield query_tmp

    f.close()


def outfmt6_to_sqlite(outfmt6_file, db_file, gzip_flag=False, tmp_pool=10000, overwrite=False):
    db_file = os.path.abspath(db_file)

    if overwrite and os.path.exists(db_file):
        os.remove(db_file)

    db_file = "sqlite:///" + db_file

    engine = create_engine(db_file)
    Base.metadata.create_all(engine)

    Session = sessionmaker(bind=engine)
    session = Session()

    query_num_start = session.query(BlastQuery).count()
    hit_num_start = session.query(BlastHit).count()
    hsp_num_start = session.query(BlastHsp).count()

    query_list_tmp = []
    for query_tmp in outfmt6_read_big(outfmt6_file, gzip_flag, query_num_start, hit_num_start, hsp_num_start):
        query_list_tmp.append(query_tmp)
        if len(query_list_tmp) >= tmp_pool:
            session.add_all(query_list_tmp)
            session.commit()
            query_list_tmp = []
    if not len(query_list_tmp) == 0:
        session.add_all(query_list_tmp)
        session.commit()

    session.close()


def get_blast_record(db_file, filter_para):
    """
    filter_para = {
        "query" : [],
        "hit" : []
    }
    """
    db_file = os.path.abspath(db_file)

    if not os.path.exists(db_file):
        raise EnvironmentError("failed found %s" % db_file)

    db_file = "sqlite:///" + db_file

    engine = create_engine(db_file)

    Session = sessionmaker(bind=engine)
    session = Session()

    query_list = session.query(BlastQuery).filter(BlastQuery.qDef.in_(filter_para['query'])).all()
    query_hsp = []
    for query in query_list:
        query_hsp.extend(query.hsp)

    subject_list = session.query(BlastHit).filter(BlastHit.hit_def.in_(filter_para['hit'])).all()
    subject_hsp = []
    for subject in subject_list:
        subject_hsp.extend(subject.hsp)

    session.close()

    if len(query_hsp) == 0:
        return subject_hsp
    elif len(subject_hsp) == 0:
        return query_hsp
    else:
        return list(set(query_hsp) & set(subject_hsp))

if __name__ == '__main__':
    outfmt6_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/WorkingDirectory/Blast1_1.txt.gz'
    db_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/WorkingDirectory/test.db'

    outfmt6_to_sqlite(outfmt6_file, db_file, gzip_flag=True, tmp_pool=10000, overwrite=True)




from api.common.genome.blast import outfmt6_read_big
from lib.xuyuxing.evolution.taxonomy import read_tax_record_dict_db
from collections import OrderedDict
from lib.common.os import multiprocess_running


def donorGenus(blast_query, donor_taxon_id, receptor_taxon_id, tax_db_file):
    """
    return (num of donor genus in top 10), (if top 1 genus is donor)
    """

    # get hit info
    hit_list = []
    all_taxon_id_list = []
    for i in blast_query.hit:
        hit_list.append((i.Hit_def, i.Hit_taxon_id[0], i.hsp[0].Hsp_bit_score))
        all_taxon_id_list.append(i.Hit_taxon_id[0])

    # get max score for each genus, exclude receptor subject
    tax_record_dict = read_tax_record_dict_db(tax_db_file, all_taxon_id_list)

    genus_score_dict = OrderedDict()
    hash_donor_genus_table = {}
    for accession, taxid, score in hit_list:
        if taxid == '' or taxid not in tax_record_dict:
            continue

        # get taxon record
        taxon_tmp = tax_record_dict[taxid]
        if hasattr(taxon_tmp, 'genus'):
            genus_id = taxon_tmp.genus
        else:
            genus_id = taxon_tmp.tax_id

        # if in donor?
        if_in_donor = taxon_tmp.is_child_of(donor_taxon_id)
        hash_donor_genus_table[genus_id] = if_in_donor

        # if in receptor, remove it
        if_in_receptor = taxon_tmp.is_child_of(receptor_taxon_id)
        if if_in_receptor:
            continue

        # keep genus max score
        if genus_id in genus_score_dict:
            continue
        else:
            genus_score_dict[genus_id] = score

    # get top 10 score genus
    top_10_genus = list(genus_score_dict.keys())[0:10]

    if len(top_10_genus) == 0:
        return (0, False)
    else:

        donor_genus_rank = []
        no_donor_genus_rank = []
        for i in top_10_genus:
            if hash_donor_genus_table[i]:
                donor_genus_rank.append(top_10_genus.index(i) + 1)
            else:
                no_donor_genus_rank.append(top_10_genus.index(i) + 1)

        # if top 1 is donor
        if hash_donor_genus_table[top_10_genus[0]]:
            return (len(donor_genus_rank), True)
        else:
            return (len(donor_genus_rank), False)


def QuickBlastHGT(taxon_outfmt6_file, donor_taxon_id, receptor_taxon_id, tax_db_file, num_threads=10, log_file=None):
    args_list = []
    args_id_list = []

    output_dict = {}

    num = 0
    for query_tmp in outfmt6_read_big(taxon_outfmt6_file,
                                      fieldname=["qseqid", "sseqid", "staxids", "pident", "length", "mismatch",
                                                 "gapopen", "qstart", "qend", "sstart", "send", "evalue",
                                                 "bitscore"]):
        num += 1
        args_list.append((query_tmp, donor_taxon_id,
                          receptor_taxon_id, tax_db_file))
        args_id_list.append(query_tmp.qDef)

        if len(args_list) >= 300:

            print(num)
            mp_out = multiprocess_running(
                donorGenus, args_list, num_threads, log_file=log_file, silence=True, args_id_list=args_id_list)
            for i in mp_out:
                output_dict[i] = mp_out[i]['output']

            args_list = []
            args_id_list = []

    mp_out = multiprocess_running(donorGenus, args_list, num_threads,
                                  log_file=log_file, silence=True, args_id_list=args_id_list)
    for i in mp_out:
        output_dict[i] = mp_out[i]['output']

    return output_dict


if __name__ == "__main__":

    class abc(object):
        pass

    args = abc()

    args.taxon_outfmt6_file = '/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/passed.diamond.taxon.bls'
    args.donor_taxon_id = '4751'
    args.receptor_taxon_id = '4747'
    args.tax_db_file = '/lustre/home/xuyuxing/Database/NCBI/taxonomy/tax_xyx.db'
    args.num_threads = 56
    args.output_file = '/lustre/home/xuyuxing/Work/Gel/HGT_WPGmapper/pseudo/HGT.txt'



    output_dict = QuickBlastHGT(args.taxon_outfmt6_file, args.donor_taxon_id, args.receptor_taxon_id, args.tax_db_file, args.num_threads)

    with open(args.output_file, 'w') as f:
        for i in output_dict:
            f.write("%s\t%d\t%s\n" % (i,output_dict[i][0],output_dict[i][1]))


import sys
from lib.common.genome.genome_feature2 import read_gff_file, write_gff_file, gene_rename, Genome, genome_rename, convert_dict_structure
from lib.common.fileIO import read_list_file
from lib.xuyuxing.base.common_command import merge_dict
from lib.common.os import cmd_run


def extract_gff_main(args):
    if not args.output_gff_file is None:
        sys.stdout = open(args.output_gff_file, 'w')

    gf_id_list = read_list_file(args.genome_feature_id_list)

    gf_dict = read_gff_file(args.raw_gff_file)

    gf_col_dict = {}
    for type_tmp in gf_dict:
        for gf_id in gf_dict[type_tmp]:
            gf_col_dict[gf_id] = gf_dict[type_tmp][gf_id]

    output_gf_list = [gf_col_dict[i] for i in gf_id_list]

    write_gff_file(output_gf_list, args.output_gff_file)


def gff_rename_main(args):
    """
    class abc():
        pass

    args = abc()
    args.input_gff_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/rename/Gel.rep.gff3'
    args.rename_prefix = "GelP"
    args.rename_chr_id = False
    args.output_gff_file = '/lustre/home/xuyuxing/Work/Gel/Pseudogene/rename/Gel.Pseudo.gff3'
    """

    if not args.rename_chr_id:

        gff_dict = read_gff_file(args.input_gff_file)

        gf_list = []
        for gf_type in gff_dict:
            for gf_id in gff_dict[gf_type]:
                gf_list.append(gff_dict[gf_type][gf_id])

        chr_dict = {}
        for gf in gf_list:
            if gf.chr_id not in chr_dict:
                chr_dict[gf.chr_id] = []
            chr_dict[gf.chr_id].append(gf)

        num = 0
        total_num = len(gf_list)
        renamed_gf_list = []
        string_format = "%s%0"+str(len(str(total_num)))+"d"
        for contig_id in sorted(list(chr_dict.keys())):
            for raw_gf in chr_dict[contig_id]:
                gene_new_name = string_format % (args.rename_prefix, num)
                # print(gene_new_name)
                new_gf = gene_rename(raw_gf, gene_new_name, contig_id)
                renamed_gf_list.append(new_gf)
                num += 1

        write_gff_file(renamed_gf_list, args.output_gff_file)

    else:

        genome = Genome(gff_file=args.input_gff_file)
        genome.genome_feature_parse()
        genome.get_chromosome_from_gff()
        genome.build_gene_sequence()

        new_genome, chr_rename_dict, gene_rename_dict = genome_rename(
            genome, args.rename_prefix, keep_raw_contig_id=False)

        gf_list = []
        for gf_type in new_genome.feature_dict:
            gf_list.extend([new_genome.feature_dict[gf_type][i]
                            for i in new_genome.feature_dict[gf_type]])

        write_gff_file(gf_list, args.output_gff_file)


def sort_gff_file(args):
    if args.sort_for_tabix:
        cmd_string = "(grep ^\"#\" %s; grep -v ^\"#\" %s | sort -k1,1 -k4,4n) > %s" % (args.input_gff_file, args.input_gff_file, args.output_gff_file)
        cmd_run(cmd_string, silence=True)

    else:

        gff_dict = read_gff_file(args.input_gff_file)
        gene_dict, chr_dict = convert_dict_structure(gff_dict)

        write_gf_id_list = []
        for contig in chr_dict:
            gf_dict = merge_dict([chr_dict[contig]['+'], chr_dict[contig]['-']], False)
            sorted_gf_list = sorted(list(gf_dict.keys()),key=lambda x:gf_dict[x].start)
            write_gf_id_list.extend(sorted_gf_list)

        write_gf_list = [gene_dict[i] for i in write_gf_id_list]

        write_gff_file(write_gf_list, args.output_gff_file)



# coding:utf-8
from cyvcf2 import VCF
from pyfaidx import Fasta
from lib.common.genome.seq_base import fancy_name_get


def get_speci_info(seq_id):
    return seq_id.split("|")[0]


if __name__ == '__main__':
    import argparse

    ###### argument parse
    parser = argparse.ArgumentParser(
        prog='SNPTools',
    )

    subparsers = parser.add_subparsers(title='subcommands', dest="subcommand_name")

    # argparse for VCFcompare
    parser_a = subparsers.add_parser('VCFcompare',
                                     help='compare two vcf file')
    parser_a.add_argument("vcf_1", help="first vcf file", type=str)
    parser_a.add_argument("vcf_2", help="second vcf file", type=str)
    parser_a.add_argument("ref", help="ref genome fasta file", type=str)
    parser_a.add_argument("-t", "--vcf_file_type", help="what type of vcf file you give? (default:deepvarient)",
                          default="deepvarient", choices=['deepvarient', 'gatk'])

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    #### command detail

    if args_dict["subcommand_name"] == "VCFcompare":
        # pre
        """
        class abc(object):
            pass

        args = abc()
        args.vcf_1 = "/lustre/home/xuyuxing/Work/Other/Guojin/201905/analysis/deepvarient/CM.output.vcf"
        args.vcf_2 = "/lustre/home/xuyuxing/Work/Other/Guojin/201905/analysis/deepvarient/Mutant.output.vcf"
        args.ref = "/lustre/home/xuyuxing/Work/Other/Guojin/201905/analysis/database/Slycopersicum_514_SL3.0.fa"
        args.vcf_file_type = "deepvarient"
        """

        ref_dict = Fasta(args.ref)

        if args.vcf_file_type == "deepvarient":

            vcf1 = VCF(args.vcf_1)
            vcf2 = VCF(args.vcf_2)

            fancy_name_get("SL3.0ch00", 1, 1000000)

            for v in vcf1(fancy_name_get("SL3.0ch00", 1, 1000000)):
                #if v.INFO["AF"] > 0.1: continue
                print(str(v))



import os
import re
import uuid
import xml.etree.ElementTree as ET
from math import log10
import gzip
from lxml import etree
import sqlite3
from lib.common.sqlite_command import sqlite_select, check_sql_table, init_sql_db_many_table, sqlite_write
from collections import OrderedDict
from lib.common.fileIO import tsv_file_dict_parse
from lib.common.util import logging_init
import lib.common.sqlite_command as sc
from lib.common.os import mkdir, cmd_run
from lib.common.math.interval import section


class BlastQuery(object):
    def __init__(self, num=None, qID=None, qDef=None, qLen=None, hit=None):
        self.num = num
        self.qID = qID
        self.qDef = qDef
        self.qLen = int(qLen) if not qLen is None else qLen
        self.hit = hit

    def __str__(self):
        short_qDef = re.search('^(\S+)', self.qDef).group(1)
        num_hit = len(self.hit)
        return "Query: %s\n" \
               "Hit_num: %d\n" % (short_qDef, num_hit)

    def all_subject(self):
        subject_id = []
        for i in self.hit:
            short_hDef = re.search('^(\S+)', i.Hit_def).group(1)
            if short_hDef in subject_id:
                continue
            else:
                subject_id.append(short_hDef)
        return subject_id

    def short_qDef(self):
        short_qDef = re.search('^(\S+)', self.qDef).group(1)
        return short_qDef


class BlastHit(object):
    def __str__(self):
        short_qDef = re.search('^(\S+)', self.query.qDef).group(1)
        short_hDef = re.search('^(\S+)', self.Hit_def).group(1)
        num_hit = len(self.hsp)
        return "Query: %s\n" \
               "Subject: %s\n" \
               "Hsp_num: %d\n" % (short_qDef, short_hDef, num_hit)

    def __init__(self, query=None, Hit_num=None, Hit_id=None, Hit_def=None, Hit_accession=None, Hit_len=None, hsp=None):
        self.query = query
        self.Hit_num = Hit_num
        self.Hit_id = Hit_id
        self.Hit_def = Hit_def
        self.Hit_accession = Hit_accession
        self.Hit_len = int(Hit_len) if not Hit_len is None else Hit_len
        self.hsp = hsp

    def short_hDef(self):
        short_hDef = re.search('^(\S+)', self.Hit_def).group(1)
        return short_hDef


class BlastHsp(object):
    def __init__(self, query, subject, Hsp_num=None, Hsp_bit_score=None, Hsp_score=None, Hsp_evalue=None,
                 Hsp_query_from=None, Hsp_query_to=None,
                 Hsp_hit_from=None, Hsp_hit_to=None, Hsp_query_frame=None, Hsp_hit_frame=None, Hsp_identity=None,
                 Hsp_positive=None, Hsp_gaps=None,
                 Hsp_align_len=None, Hsp_qseq=None, Hsp_hseq=None, Hsp_midline=None):
        self.query = query
        self.subject = subject
        self.Hsp_num = int(Hsp_num)
        self.Hsp_bit_score = float(Hsp_bit_score)
        if Hsp_score is None:
            self.Hsp_score = None
        else:
            self.Hsp_score = int(Hsp_score)
        self.Hsp_evalue = float(Hsp_evalue)
        self.Hsp_query_from = int(Hsp_query_from)
        self.Hsp_query_to = int(Hsp_query_to)
        self.Hsp_hit_from = int(Hsp_hit_from)
        self.Hsp_hit_to = int(Hsp_hit_to)
        if Hsp_query_frame is None:
            self.Hsp_query_frame = None
        else:
            self.Hsp_query_frame = int(Hsp_query_frame)
        if Hsp_hit_frame is None:
            self.Hsp_hit_frame = None
        else:
            self.Hsp_hit_frame = int(Hsp_hit_frame)

        try:
            self.Hsp_identical_site = int(Hsp_identity)
            self.Hsp_identical_ratio = int(Hsp_identity)/int(Hsp_align_len)
        except:
            self.Hsp_identical_ratio = float(Hsp_identity)/100
            self.Hsp_identical_site = self.Hsp_identical_ratio * int(Hsp_align_len)

        if Hsp_positive is None:
            self.Hsp_positive = None
        else:
            self.Hsp_positive = int(Hsp_positive)
        self.Hsp_gaps = int(Hsp_gaps)
        self.Hsp_align_len = int(Hsp_align_len)
        if Hsp_qseq is None:
            self.Hsp_qseq = None
        else:
            self.Hsp_qseq = str(Hsp_qseq)
        if Hsp_hseq is None:
            self.Hsp_hseq = None
        else:
            self.Hsp_hseq = str(Hsp_hseq)
        if Hsp_midline is None:
            self.Hsp_midline = None
        else:
            self.Hsp_midline = str(Hsp_midline)


# blast output parser

def fast_iter(context, func, *args, **kwargs):
    """
    http://lxml.de/parsing.html#modifying-the-tree
    Based on Liza Daly's fast_iter
    http://www.ibm.com/developerworks/xml/library/x-hiperfparse/
    See also http://effbot.org/zone/element-iterparse.htm
    """
    for event, elem in context:
        func(elem, *args, **kwargs)
        # It's safe to call clear() here because no descendants will be
        # accessed
        elem.clear()
        # Also eliminate now-empty references from the root node to elem
        for ancestor in elem.xpath('ancestor-or-self::*'):
            while ancestor.getprevious() is not None:
                del ancestor.getparent()[0]
    del context


def outfmt5_complete(input_file):
    from lib.xuyuxing.base.base_function import cmd_run
    output = cmd_run('tail -2 %s' % input_file, silence=True)
    if output[0] is True and output[1] == "</BlastOutput>\n\n":
        return True
    else:
        return False


def outfmt5_parameters(input_file):
    doc = etree.iterparse(input_file, events=('start', 'end'))
    _, root = next(doc)
    start_tag = None
    output_dict = {}
    for event, element in doc:
        if event == 'start' and start_tag is None:
            start_tag = element.tag
        if event == 'end' and element.tag == start_tag:
            if element.tag == 'BlastOutput_program':
                output_dict['program'] = element.text
            elif element.tag == 'BlastOutput_db':
                output_dict['db'] = element.text
            elif element.tag == 'BlastOutput_param':
                for params in element:
                    for param in params:
                        output_dict[re.sub(
                            'Parameters_', '', param.tag)] = param.text
                break
            start_tag = None
            root.clear()
    return output_dict


def outfmt5_read(input_file):
    tree = ET.parse(input_file)
    root = tree.getroot()
    output_dict = {}
    for Iteration in root.iter('Iteration'):
        Iteration_iter_num = Iteration.find('Iteration_iter-num').text
        Iteration_query_ID = Iteration.find('Iteration_query-ID').text
        Iteration_query_def = Iteration.find('Iteration_query-def').text
        Iteration_query_len = Iteration.find('Iteration_query-len').text
        query_tmp = BlastQuery(Iteration_iter_num, Iteration_query_ID,
                               Iteration_query_def, Iteration_query_len, [])
        # if Iteration.find('Iteration_message') is not None:
        #    if Iteration.find('Iteration_message').text == "No hits found":
        #        output_dict[Iteration_query_def] = query_tmp
        #        continue
        #    else:
        #        print Iteration.find('Iteration_message').text
        for Hit in Iteration.iter('Hit'):
            Hit_num = Hit.find('Hit_num').text
            Hit_id = Hit.find('Hit_id').text
            Hit_def = Hit.find('Hit_def').text
            Hit_accession = Hit.find('Hit_accession').text
            Hit_len = Hit.find('Hit_len').text
            Hit_tmp = BlastHit(query_tmp, Hit_num, Hit_id,
                               Hit_def, Hit_accession, Hit_len, [])
            for Hsp in Hit.iter('Hsp'):
                Hsp_num = Hsp.find('Hsp_num').text
                Hsp_bit_score = Hsp.find('Hsp_bit-score').text
                Hsp_score = Hsp.find('Hsp_score').text
                Hsp_evalue = Hsp.find('Hsp_evalue').text
                Hsp_query_from = Hsp.find('Hsp_query-from').text
                Hsp_query_to = Hsp.find('Hsp_query-to').text
                Hsp_hit_from = Hsp.find('Hsp_hit-from').text
                Hsp_hit_to = Hsp.find('Hsp_hit-to').text
                Hsp_query_frame = Hsp.find('Hsp_query-frame').text
                Hsp_hit_frame = Hsp.find('Hsp_hit-frame').text
                Hsp_identity = Hsp.find('Hsp_identity').text
                Hsp_positive = Hsp.find('Hsp_positive').text
                Hsp_gaps = Hsp.find('Hsp_gaps').text
                Hsp_align_len = Hsp.find('Hsp_align-len').text
                Hsp_qseq = Hsp.find('Hsp_qseq').text
                Hsp_hseq = Hsp.find('Hsp_hseq').text
                Hsp_midline = Hsp.find('Hsp_midline').text
                Hsp_tmp = BlastHsp(query_tmp, Hit_tmp, Hsp_num, Hsp_bit_score, Hsp_score, Hsp_evalue, Hsp_query_from,
                                   Hsp_query_to, Hsp_hit_from, Hsp_hit_to, Hsp_query_frame, Hsp_hit_frame, Hsp_identity,
                                   Hsp_positive, Hsp_gaps, Hsp_align_len, Hsp_qseq, Hsp_hseq, Hsp_midline)
                Hit_tmp.hsp.append(Hsp_tmp)
            query_tmp.hit.append(Hit_tmp)
        output_dict[Iteration_query_def] = query_tmp
    return output_dict


def outfmt5_read_big(input_file, keep_no_hits=False):
    context = etree.iterparse(input_file, tag="Iteration")
    for event, Iteration in context:
        Iteration_iter_num = Iteration.find('Iteration_iter-num').text
        Iteration_query_ID = Iteration.find('Iteration_query-ID').text
        Iteration_query_def = Iteration.find('Iteration_query-def').text
        Iteration_query_len = Iteration.find('Iteration_query-len').text
        query_tmp = BlastQuery(Iteration_iter_num, Iteration_query_ID,
                               Iteration_query_def, Iteration_query_len, [])
        if not (Iteration.find('Iteration_message') is not None and Iteration.find(
                'Iteration_message').text == "No hits found" and keep_no_hits is False):
            for Hit in Iteration.iter('Hit'):
                Hit_num = Hit.find('Hit_num').text
                Hit_id = Hit.find('Hit_id').text
                Hit_def = Hit.find('Hit_def').text
                Hit_accession = Hit.find('Hit_accession').text
                Hit_len = Hit.find('Hit_len').text
                Hit_tmp = BlastHit(query_tmp, Hit_num, Hit_id,
                                   Hit_def, Hit_accession, Hit_len, [])
                for Hsp in Hit.iter('Hsp'):
                    Hsp_num = Hsp.find('Hsp_num').text
                    Hsp_bit_score = Hsp.find('Hsp_bit-score').text
                    Hsp_score = Hsp.find('Hsp_score').text
                    Hsp_evalue = Hsp.find('Hsp_evalue').text
                    Hsp_query_from = Hsp.find('Hsp_query-from').text
                    Hsp_query_to = Hsp.find('Hsp_query-to').text
                    Hsp_hit_from = Hsp.find('Hsp_hit-from').text
                    Hsp_hit_to = Hsp.find('Hsp_hit-to').text
                    Hsp_query_frame = Hsp.find('Hsp_query-frame').text
                    Hsp_hit_frame = Hsp.find('Hsp_hit-frame').text
                    Hsp_identity = Hsp.find('Hsp_identity').text
                    Hsp_positive = Hsp.find('Hsp_positive').text
                    Hsp_gaps = Hsp.find('Hsp_gaps').text
                    Hsp_align_len = Hsp.find('Hsp_align-len').text
                    Hsp_qseq = Hsp.find('Hsp_qseq').text
                    Hsp_hseq = Hsp.find('Hsp_hseq').text
                    Hsp_midline = Hsp.find('Hsp_midline').text
                    print(Hsp_identity)
                    Hsp_tmp = BlastHsp(query_tmp, Hit_tmp, Hsp_num, Hsp_bit_score, Hsp_score, Hsp_evalue,
                                       Hsp_query_from,
                                       Hsp_query_to, Hsp_hit_from, Hsp_hit_to, Hsp_query_frame, Hsp_hit_frame,
                                       Hsp_identity,
                                       Hsp_positive, Hsp_gaps, Hsp_align_len, Hsp_qseq, Hsp_hseq, Hsp_midline)
                    Hit_tmp.hsp.append(Hsp_tmp)
                query_tmp.hit.append(Hit_tmp)
            # print(query_tmp.qDef)
            yield query_tmp
        # It's safe to call clear() here because no descendants will be
        # accessed
        Iteration.clear()
        # Also eliminate now-empty references from the root node to elem
        for ancestor in Iteration.xpath('ancestor-or-self::*'):
            while ancestor.getprevious() is not None:
                del ancestor.getparent()[0]
    del context
    # fast_iter(context, iteration_parser, keep_no_hits=keep_no_hits)


def outfmt6_read(input_file, gzip_flag=False):
    fieldname = ["q_id", "s_id", "identity", "aln_len", "mis", "gap", "q_start", "q_end", "s_start", "s_end", "e_value",
                 "score"]
    all_file = tsv_file_dict_parse(
        input_file, fieldnames=fieldname, gzip_flag=gzip_flag)
    query_dir = {}
    for i in all_file:
        q_id = all_file[i]["q_id"]
        s_id = all_file[i]["s_id"]
        if q_id not in query_dir:
            query_dir[q_id] = {"subject_key_list": []}
        if s_id not in query_dir[q_id]:
            query_dir[q_id][s_id] = []
            query_dir[q_id]["subject_key_list"].append(s_id)
        query_dir[q_id][s_id].append(all_file[i])
    output_dict = {}
    num = 0
    for query in query_dir:
        num = num + 1
        query_tmp = BlastQuery(num, query, None, None, [])
        hit_num = 0
        for hit in query_dir[query]["subject_key_list"]:
            hit_num = hit_num + 1
            hit_tmp = BlastHit(query_tmp, hit_num, hit, None, None, None, [])
            hsp_num = 0
            for hsp in query_dir[query][hit]:
                hsp_num = hsp_num + 1
                hsp_tmp = BlastHsp(query_tmp, hit_tmp, hsp_num, hsp["score"], None, hsp["e_value"], hsp["q_start"],
                                   hsp["q_end"], hsp["s_start"], hsp["s_end"], None, None, hsp["identity"], None,
                                   hsp["gap"],
                                   hsp["aln_len"], None, None, None)
                hit_tmp.hsp.append(hsp_tmp)
            query_tmp.hit.append(hit_tmp)
        output_dict[query] = query_tmp
    return output_dict


def outfmt6_read_big(input_file,
                     fieldname=["qseqid", "sseqid", "pident", "length", "mismatch", "gapopen", "qstart", "qend",
                                "sstart", "send", "evalue", "bitscore"], gzip_flag=False):
    if gzip_flag:
        f = gzip.open(input_file, 'rt')
    else:
        f = open(input_file, 'r')

    query_num = 0
    query_tmp = None
    for each_line in f:
        each_line = each_line.strip()
        info = each_line.split("\t")

        info_dict = {}
        for i in range(len(fieldname)):
            info_dict[fieldname[i]] = info[i]

        if query_tmp is None:
            new_query_flag = 1
            new_hit_flag = 1
        else:
            last_query_def = query_tmp.qDef
            last_hit_def = query_tmp.hit[-1].Hit_def
            if last_query_def != info_dict['qseqid']:
                new_query_flag = 1
                new_hit_flag = 1
            else:
                new_query_flag = 0
                if last_hit_def != info_dict['sseqid']:
                    new_hit_flag = 1
                else:
                    new_hit_flag = 0

        if new_query_flag:
            if query_tmp is not None:
                yield query_tmp

            query_num = query_num + 1
            query_tmp = BlastQuery(
                query_num, info_dict['qseqid'], info_dict['qseqid'], None, [])
            hit_num = 0

        if new_hit_flag:
            hit_num = hit_num + 1
            hit_tmp = BlastHit(query_tmp, hit_num, info_dict['sseqid'], info_dict['sseqid'], info_dict['sseqid'], None,
                               [])
            if "staxids" in fieldname:
                hit_tmp.Hit_taxon_id = info_dict['staxids'].split(";")

            query_tmp.hit.append(hit_tmp)
            hsp_num = 0

        hsp_num = hsp_num + 1
        hsp_tmp = BlastHsp(query_tmp, query_tmp.hit[-1], hsp_num, float(info_dict['bitscore']), None,
                           float(info_dict['evalue']), int(
                               info_dict['qstart']),
                           int(info_dict['qend']), int(info_dict['sstart']), int(
                               info_dict['send']), None, None,
                           info_dict['pident'], None, int(
                               info_dict['gapopen']),
                           int(info_dict['length']), None, None, None)
        hsp_tmp.Hsp_mismatch = float(info_dict['mismatch'])

        query_tmp.hit[-1].hsp.append(hsp_tmp)

    if query_tmp is not None:
        yield query_tmp

    f.close()


def outfmt6_write(output_dict_from_outfmt6):
    for query in output_dict_from_outfmt6:
        for subject in output_dict_from_outfmt6[query].hit:
            for hsp in subject.hsp:
                query_id = output_dict_from_outfmt6[query].short_qDef()
                subject_id = subject.short_hDef()
                identity = float(hsp.Hsp_identity) / \
                    float(hsp.Hsp_align_len) * 100
                alignment_length = hsp.Hsp_align_len
                mismatches = hsp.Hsp_align_len - hsp.Hsp_identity
                gap_openings = hsp.Hsp_gaps
                q_start = hsp.Hsp_query_from
                q_end = hsp.Hsp_query_to
                s_start = hsp.Hsp_hit_from
                s_end = hsp.Hsp_hit_to
                e_value = hsp.Hsp_evalue

                outfmt6_print = "%s\t%s\t%.2f\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%.5e\t%.2f" % \
                                (query_id, subject_id, identity, alignment_length, mismatches,
                                 gap_openings, q_start, q_end, s_start, s_end, e_value, hsp.Hsp_bit_score)

                yield outfmt6_print


def keep_outfmt6_info(query_record, fancy_name=False, fancy_name_type=1):
    for subject in query_record.hit:
        for hsp in subject.hsp:
            query_id = query_record.short_qDef()
            if fancy_name is True:
                if fancy_name_type == 1:
                    query_id, c_start, c_end = re.search(
                        r'^(\S+):(\d+)-(\d+)$', query_id).groups()
                elif fancy_name_type == 2:
                    query_id, c_start, c_end = re.search(
                        r'^(\S+)_(\d+)-(\d+)$', query_id).groups()
                q_start = int(c_start) + hsp.Hsp_query_from - 1
                q_end = int(c_start) + hsp.Hsp_query_to - 1
            else:
                q_start = hsp.Hsp_query_from
                q_end = hsp.Hsp_query_to
            subject_id = subject.short_hDef()
            identity = float(hsp.Hsp_identity) / float(hsp.Hsp_align_len) * 100
            alignment_length = hsp.Hsp_align_len
            mismatches = hsp.Hsp_align_len - hsp.Hsp_identity
            gap_openings = hsp.Hsp_gaps
            s_start = hsp.Hsp_hit_from
            s_end = hsp.Hsp_hit_to
            e_value = hsp.Hsp_evalue

            outfmt6_print = (query_id, subject_id, identity, alignment_length, mismatches,
                             gap_openings, q_start, q_end, s_start, s_end, e_value, hsp.Hsp_bit_score)

            yield outfmt6_print


# store blast results to sqlite

def blast_to_sqlite(db_name, bls_file, query_fasta=None, subject_fasta=None, outfmt=6, hsp_keep=None, hit_keep=None,
                    log_file=None, fancy_name=False, gzip_flag=False):
    """
    take blast into sqlite file

    sqlite3 structure
    tables

    1. query
        query_num: query rank id
        qDef: query name in blast results
        qID: query name ID in blast results
        qLen: query seq length
        q_hsp_id: a comma split number string to store hsp_id which query is this query record

    2. subject
        hit_num: subject rank_id
        hit_def: subject name in blast results
        hit_id: subject name ID in blast results
        hit_len: subject seq length
        hit_accession: subject seq accession
        s_hsp_id: a comma split number string to store hsp_id which subject is this subject record

    3. hsp
        hsp_id: hsp rank id
        query_num: query rank id
        hit_num: subject rank id
        query_id: query name ID in blast results
        hit_id: subject name ID in blast results
        hit_rank
        Hsp_num
        Hsp_bit_score
        Hsp_score
        Hsp_evalue
        Hsp_query_from
        Hsp_query_to
        Hsp_hit_from
        Hsp_hit_to
        Hsp_query_frame
        Hsp_hit_frame
        Hsp_identity
        Hsp_positive
        Hsp_gaps
        Hsp_align_len
        Hsp_qseq
        Hsp_hseq
        Hsp_midline

    db_name='/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pt_file/OrthoFinder/Results_Aug01/WorkingDirectory/Blast39_23.db.txt'
    bls_file='/lustre/home/xuyuxing/Work/Gel/Gene_Loss/plant/pt_file/OrthoFinder/Results_Aug01/WorkingDirectory/Blast39_23.txt.gz'
    query_fasta=None
    subject_fasta=None
    outfmt=6
    hsp_keep=None
    hit_keep=None
    log_file=None
    fancy_name=False
    gzip_flag=True

    """

    # give column name
    query_columns = ["query_num", "qDef", "qID", "qLen", "q_hsp_id"]

    subject_columns = ["hit_num", "hit_def", "hit_id",
                       "hit_len", "hit_accession", "s_hsp_id"]

    hsp_columns = ["hsp_id", "query_num", "hit_num", "query_id", "hit_id", "hit_rank", "Hsp_num", "Hsp_bit_score",
                   "Hsp_score", "Hsp_evalue", "Hsp_query_from", "Hsp_query_to", "Hsp_hit_from", "Hsp_hit_to",
                   "Hsp_query_frame", "Hsp_hit_frame", "Hsp_identity", "Hsp_positive", "Hsp_gaps", "Hsp_align_len",
                   "Hsp_qseq", "Hsp_hseq", "Hsp_midline"]

    # begin
    module_log = logging_init("blast_to_sqlite", log_file)
    module_log.info(
        'received a call to load bls file %s into sqlite db %s' % (bls_file, db_name))

    # init database
    init_sql_db_many_table(db_name, {
                           "query": query_columns, "subject": subject_columns, "hsp": hsp_columns}, True)

    # load blast results
    if outfmt == 6:
        bls_file_parser = outfmt6_read_big(bls_file, gzip_flag=gzip_flag)
    elif outfmt == 5:
        bls_file_parser = outfmt5_read_big(bls_file)
    else:
        raise ValueError("unknown format")

    waitting_hsp = []
    stored_subject = {}
    stored_query = {}
    hsp_id = 0
    for query in bls_file_parser:
        # store query

        # parse fancy name
        if fancy_name is True:
            query_true_name, c_start, c_end = re.search(
                r'^(\S+):(\d+)-(\d+)$', query.short_qDef()).groups()
            query_info = [len(stored_query) + 1, query_true_name,
                          query_true_name, query.qLen]
        else:
            query_info = [len(stored_query) + 1, query.qDef,
                          query.qID, query.qLen]

        q_def = query_info[1]
        if not q_def in stored_query:
            stored_query[q_def] = {
                "info": query_info,
                "hsp_list": []
            }

        for subject in query.hit:
            # make sure subject is need to store
            if not hit_keep is None and subject.Hit_num > hit_keep:
                break

            # store subject info
            subject_info = [len(stored_subject) + 1, subject.Hit_def, subject.Hit_id, subject.Hit_len,
                            subject.Hit_accession]

            s_def = subject_info[1]
            if not s_def in stored_subject:
                stored_subject[s_def] = {
                    "info": subject_info,
                    "hsp_list": []
                }

            # get hsp info and store
            for hsp in subject.hsp:
                # make sure subject is need to store
                if not hsp_keep is None and hsp.Hsp_num > hsp_keep:
                    break

                # parse fancy name
                if fancy_name is True:
                    query_true_name, c_start, c_end = re.search(
                        r'^(\S+):(\d+)-(\d+)$', query.short_qDef()).groups()
                    q_start = int(c_start) + hsp.Hsp_query_from - 1
                    q_end = int(c_start) + hsp.Hsp_query_to - 1
                else:
                    q_start = hsp.Hsp_query_from
                    q_end = hsp.Hsp_query_to

                hsp_id = hsp_id + 1

                hsp_info = (
                    hsp_id, stored_query[q_def]["info"][0], stored_subject[s_def]["info"][0], q_def, s_def,
                    subject.Hit_num,
                    hsp.Hsp_num, hsp.Hsp_bit_score, hsp.Hsp_score, hsp.Hsp_evalue, q_start, q_end, hsp.Hsp_hit_from,
                    hsp.Hsp_hit_to, hsp.Hsp_query_frame, hsp.Hsp_hit_frame, hsp.Hsp_identity, hsp.Hsp_positive,
                    hsp.Hsp_gaps, hsp.Hsp_align_len, hsp.Hsp_qseq, hsp.Hsp_hseq, hsp.Hsp_midline)

                stored_query[q_def]["hsp_list"].append(hsp_id)
                stored_subject[s_def]["hsp_list"].append(hsp_id)

                waitting_hsp.append(hsp_info)

            if len(waitting_hsp) > 10000:
                sqlite_write(waitting_hsp, db_name, "hsp", hsp_columns)
                waitting_hsp = []
                module_log.info("load %d record" % hsp_id)

    if not len(waitting_hsp) == 0:
        sqlite_write(waitting_hsp, db_name, "hsp", hsp_columns)
        module_log.info("load %d record" % hsp_id)

    from lib.common.util import printer_list

    # write query table
    module_log.info("load query table")
    waitting_query = []
    for q_def in stored_query:
        tmp_info = stored_query[q_def]['info']
        hsp_list_tmp = stored_query[q_def]['hsp_list']
        hsp_list_str = printer_list(hsp_list_tmp, ",")
        tmp_info.append(hsp_list_str)
        waitting_query.append(tmp_info)
    sqlite_write(waitting_query, db_name, "query", query_columns)

    # write subject table
    module_log.info("load subject table")
    waitting_subject = []
    for s_def in stored_subject:
        tmp_info = stored_subject[s_def]['info']
        hsp_list_tmp = stored_subject[s_def]['hsp_list']
        hsp_list_str = printer_list(hsp_list_tmp, ",")
        tmp_info.append(hsp_list_str)
        waitting_subject.append(tmp_info)
    sqlite_write(waitting_subject, db_name, "subject", subject_columns)

    conn = sqlite3.connect(db_name)
    conn.execute("CREATE UNIQUE INDEX %s on %s (\"%s\")" %
                 ("query_index", "query", "qDef"))
    conn.execute("CREATE UNIQUE INDEX %s on %s (\"%s\")" %
                 ("subject_index", "subject", "hit_def"))
    conn.execute("CREATE UNIQUE INDEX %s on %s (\"%s\")" %
                 ("hsp_index", "hsp", "hsp_id"))
    conn.close()

    del waitting_hsp
    del waitting_subject
    del waitting_query
    module_log.info("Finished")
    del module_log.handlers[:]

    # module_log = logging_init("store_bls_in_sqlite", log_file)
    # module_log.info('received a call to "store_bls_in_sqlite"')
    #
    # # making a new sql database for store sequences
    # module_log.info('check the database for if have the tables already')
    # db_info = sc.all_database_stat(db_name)
    # prefix_matched_table = []
    # for table_name in db_info:
    #     if re.match('^%s.*' % table_name_prefix, table_name):
    #         prefix_matched_table.append(table_name)
    # if len(prefix_matched_table) == 0:
    #     module_log.info('need a new tables to store blast results')
    # elif len(prefix_matched_table) > 0:
    #     module_log.info('there are prefix matched tables in the database already, drop them')
    #     conn = sqlite3.connect(db_name)
    #     for i in prefix_matched_table:
    #         conn.execute("DROP TABLE %s" % i)
    #     conn.close()
    #
    # # loading fasta file and store to sqlite
    # module_log.info('loading blast results file and storing in sqlite')
    # record_list = []
    # table_id = 0
    # record_id = 0
    # output_dict = {}
    #
    # if outfmt == 6:
    #     bls_file_parser = outfmt6_read_big(bls_file, gzip_flag)
    # elif outfmt == 5:
    #     bls_file_parser = outfmt5_read_big(bls_file)
    # else:
    #     raise ValueError("unknown format")
    #
    # for query in bls_file_parser:
    #     hit_num = 0
    #     for subject in query.hit:
    #         hit_num = hit_num + 1
    #         if not hit_keep is None and hit_num > hit_keep:
    #             break
    #         hsp_num = 0
    #         for hsp in subject.hsp:
    #             hsp_num = hsp_num + 1
    #             if not hsp_keep is None and hsp_num > hsp_keep:
    #                 break
    #             query_short_name = query.short_qDef()
    #             query_name = query.qDef
    #             if fancy_name is True:
    #                 query_short_name, c_start, c_end = re.search(r'^(\S+):(\d+)-(\d+)$', query_short_name).groups()
    #                 q_start = int(c_start) + hsp.Hsp_query_from - 1
    #                 q_end = int(c_start) + hsp.Hsp_query_to - 1
    #             else:
    #                 q_start = hsp.Hsp_query_from
    #                 q_end = hsp.Hsp_query_to
    #             subject_short_name = subject.short_hDef()
    #             subject_name = subject.Hit_def
    #             record_id = record_id + 1
    #             record_list.append((
    #                 record_id, query_short_name, query_name, q_start, q_end, subject_short_name, subject_name,
    #                 hsp.Hsp_hit_from, hsp.Hsp_hit_to, hsp.Hsp_align_len, hsp.Hsp_gaps, hsp.Hsp_hit_frame,
    #                 hsp.Hsp_identity, hsp.Hsp_positive, hsp.Hsp_query_frame, hsp.Hsp_score, hsp.Hsp_bit_score,
    #                 hsp.Hsp_evalue, hsp.Hsp_hseq, hsp.Hsp_qseq, hsp.Hsp_midline))
    #
    #             if len(record_list) == table_max_rows:
    #                 table_name = table_name_prefix + "hsp_" + str(table_id)
    #                 table_name, index_name = add_hsp_record(record_list, db_name, table_name, outfmt5_columns)
    #                 output_dict[table_name] = {
    #                     "index": index_name,
    #                     "record_num": len(record_list)
    #                 }
    #                 module_log.info(
    #                     "\tAdd %d record in table: %s with index %s" % (len(record_list), table_name, index_name))
    #                 table_id = table_id + 1
    #                 record_list = []
    #
    # if len(record_list) > 0:
    #     table_name = table_name_prefix + "hsp_" + str(table_id)
    #     table_name, index_name = add_hsp_record(record_list, db_name, table_name, outfmt5_columns)
    #     output_dict[table_name] = {
    #         "index": index_name,
    #         "record_num": len(record_list)
    #     }
    #     module_log.info(
    #         "\tAdd %d record in table: %s with index %s" % (len(record_list), table_name, index_name))
    #     table_id = table_id + 1
    #     record_list = []
    #
    # module_log.info('loaded blast file and stored in sqlite ')
    #
    # del module_log.handlers[:]
    #
    # return output_dict


def sqlite_read(db_name):
    query_tmp = None

    for hsp_info in sqlite_select(db_name, 'hsp'):

        if query_tmp is None:
            new_query_flag = 1
            new_hit_flag = 1
        else:
            last_query_def = query_tmp.qDef
            last_hit_def = query_tmp.hit[-1].Hit_def
            if last_query_def != hsp_info[3]:
                new_query_flag = 1
                new_hit_flag = 1
            else:
                new_query_flag = 0
                if last_hit_def != hsp_info[4]:
                    new_hit_flag = 1
                else:
                    new_hit_flag = 0

        if new_query_flag:
            if query_tmp is not None:
                yield query_tmp

            query_num = hsp_info[1]
            query_info = sqlite_select(
                db_name, "query", key_name='query_num', value_tuple=(query_num,))[0]
            query_tmp = BlastQuery(
                query_info[0], query_info[1], query_info[2], query_info[3], [])

        if new_hit_flag:
            hit_num = hsp_info[2]
            hit_info = sqlite_select(
                db_name, "subject", key_name='hit_num', value_tuple=(hit_num,))[0]
            hit_tmp = BlastHit(
                query_tmp, hit_info[0], hit_info[2], hit_info[1], hit_info[4], hit_info[3], [])
            query_tmp.hit.append(hit_tmp)

        hsp_tmp = BlastHsp(query_tmp, hit_tmp, hsp_info[6], hsp_info[7], hsp_info[8], hsp_info[9], hsp_info[10],
                           hsp_info[11], hsp_info[12], hsp_info[13], hsp_info[14], hsp_info[15], hsp_info[
                               16], hsp_info[17], hsp_info[18], hsp_info[19], hsp_info[20], hsp_info[21],
                           hsp_info[22])

        query_tmp.hit[-1].hsp.append(hsp_tmp)

    if query_tmp is not None:
        yield query_tmp


# blast running

def blastn_running(query_file, db_file, blast_out_file, evalue, num_threads, realy_run=True, blast_dir="", outfmt=5,
                   task="blastn"):
    query_file = os.path.abspath(query_file)
    query_file_dir = os.path.dirname(query_file)

    db_file = os.path.abspath(db_file)
    db_file_dir = os.path.dirname(db_file)

    blast_out_file = os.path.abspath(blast_out_file)
    blast_out_file_dir = os.path.dirname(blast_out_file)

    # makeblastdb
    if (not ((os.path.exists(db_file + ".nhr") and os.path.exists(db_file + ".nin") and os.path.exists(
            db_file + ".nsq")))) and (not os.path.exists(db_file + ".nal")):
        cmd_string = blast_dir + "makeblastdb -in %s -dbtype nucl" % db_file
        cmd_run(cmd_string, cwd=db_file_dir, silence=True)

    # blastn
    cmd_string = blast_dir + "blastn -query %s -db %s -out %s -outfmt %d -task %s -evalue %s -num_threads %d" % (
        query_file, db_file, blast_out_file, outfmt, task, evalue, num_threads)

    if realy_run:
        cmd_run(cmd_string, cwd=blast_out_file_dir, silence=True)
    else:
        return cmd_string


def blastn_pair_running(query, hit, dir, keep_file=True):
    mkdir(dir, True)
    query_file = os.path.abspath(dir + "/" + uuid.uuid1().hex)
    subject_file = os.path.abspath(dir + "/" + uuid.uuid1().hex)
    blast_out_file = os.path.abspath(dir + "/" + uuid.uuid1().hex)
    with open(query_file, "w") as f:
        f.write(">" + query[0] + "\n" + query[1])
    with open(subject_file, "w") as f:
        f.write(">" + hit[0] + "\n" + hit[1])
    cmd_string = "makeblastdb -in %s -dbtype nucl" % subject_file
    cmd_run(cmd_string, cwd=dir, silence=True)
    cmd_string = "blastn -query %s -db %s -out %s -outfmt 5 -task blastn -evalue 1e-10 " % (
        query_file, subject_file, blast_out_file)
    cmd_run(cmd_string, cwd=dir, silence=True)

    if keep_file is False:
        cmd_string = "rm %s" % subject_file
        cmd_run(cmd_string, cwd=dir, silence=True)
        cmd_string = "rm %s.*" % subject_file
        cmd_run(cmd_string, cwd=dir, silence=True)
        os.remove(query_file)

    if outfmt5_complete(blast_out_file):
        output = outfmt5_read(blast_out_file)
        os.remove(blast_out_file)
        return output
    else:
        raise ValueError("Failed blast")


def evalue_to_wvalue(evalue):
    if evalue == 0.0:
        return 100
    else:
        return min(100, round(-log10(evalue) / 2))


def hit_CIP(blast_hit):
    """
    CIP (cumulative identity percentage) corresponds to the cumulative percent of sequence identity obtained for all the HSPs   
    """

    identity_list = []
    hsp_align_len = []
    for hsp in blast_hit.hsp:
        identity_list.append(hsp.Hsp_identical_site)
        hsp_align_len.append(hsp.Hsp_align_len)

    return sum(identity_list)/sum(hsp_align_len)

def hit_CALP(blast_hit):
    """
    CALP is the cumulative alignment length percentage. It represents the sum of the HSP lengths (AL) for all the HSPs divided by the length of the query sequence (CALP = AL/Query length)
    """

    
    hsp_align_len = []
    for hsp in blast_hit.hsp:
        hsp_align_len.append(hsp.Hsp_align_len)

    return sum(hsp_align_len)/blast_hit.query.qLen

def blastp_running(query_file, db_file, blast_out_file, evalue, num_threads, realy_run=True, blast_dir="", outfmt=5):
    query_file = os.path.abspath(query_file)
    query_file_dir = os.path.dirname(query_file)

    db_file = os.path.abspath(db_file)
    db_file_dir = os.path.dirname(db_file)

    blast_out_file = os.path.abspath(blast_out_file)
    blast_out_file_dir = os.path.dirname(blast_out_file)

    # makeblastdb
    if (not ((os.path.exists(db_file + ".phr") and os.path.exists(db_file + ".pin") and os.path.exists(
            db_file + ".psq")))):
        cmd_string = blast_dir + "makeblastdb -in %s -dbtype prot" % db_file
        cmd_run(cmd_string, cwd=db_file_dir, silence=True)

    # blastn
    cmd_string = blast_dir + "blastp -query %s -db %s -out %s -outfmt %d -evalue %s -num_threads %d" % (
        query_file, db_file, blast_out_file, outfmt, evalue, num_threads)

    if realy_run:
        cmd_run(cmd_string, cwd=blast_out_file_dir, silence=True)
    else:
        return cmd_string


def blastx_running(query_file, db_file, blast_out_file, evalue, num_threads, realy_run=True, blast_dir="", outfmt=5):
    query_file = os.path.abspath(query_file)
    query_file_dir = os.path.dirname(query_file)

    db_file = os.path.abspath(db_file)
    db_file_dir = os.path.dirname(db_file)

    blast_out_file = os.path.abspath(blast_out_file)
    blast_out_file_dir = os.path.dirname(blast_out_file)

    # makeblastdb
    if (not ((os.path.exists(db_file + ".phr") and os.path.exists(db_file + ".pin") and os.path.exists(
            db_file + ".psq")))):
        cmd_string = blast_dir + "makeblastdb -in %s -dbtype prot" % db_file
        cmd_run(cmd_string, cwd=db_file_dir, silence=True)

    # blastn
    cmd_string = blast_dir + "blastx -query %s -db %s -out %s -outfmt %d -evalue %s -num_threads %d" % (
        query_file, db_file, blast_out_file, outfmt, evalue, num_threads)

    if realy_run:
        cmd_run(cmd_string, cwd=blast_out_file_dir, silence=True)
    else:
        return cmd_string


def tblastn_running(query_file, db_file, blast_out_file, evalue, num_threads, realy_run=True, blast_dir="", outfmt=5):
    query_file = os.path.abspath(query_file)
    query_file_dir = os.path.dirname(query_file)

    db_file = os.path.abspath(db_file)
    db_file_dir = os.path.dirname(db_file)

    blast_out_file = os.path.abspath(blast_out_file)
    blast_out_file_dir = os.path.dirname(blast_out_file)

    # makeblastdb
    if (not ((os.path.exists(db_file + ".phr") and os.path.exists(db_file + ".pin") and os.path.exists(
            db_file + ".psq")))):
        cmd_string = blast_dir + "makeblastdb -in %s -dbtype prot" % db_file
        cmd_run(cmd_string, cwd=db_file_dir, silence=True)

    # blastn
    cmd_string = blast_dir + "tblastn -query %s -db %s -out %s -outfmt %d -evalue %s -num_threads %d" % (
        query_file, db_file, blast_out_file, outfmt, evalue, num_threads)

    if realy_run:
        cmd_run(cmd_string, cwd=blast_out_file_dir, silence=True)
    else:
        return cmd_string

# other


def NR_blast_annotation_extractor(nr_blast_results_file):
    blast_results = outfmt5_read(nr_blast_results_file)
    output_dict = {}
    for query_blast in blast_results:
        query_blast = blast_results[query_blast]
        query_name = query_blast.qDef
        query_name = query_name.split()[0]
        function_anno = []
        for hits in query_blast.hit:
            hits_def = hits.Hit_def
            # print(hits_def)
            if re.match('(.*)\[(.*)\]', hits_def):
                anno, speci = re.findall('(.*)\[(.*)\]', hits_def)[0]
                anno = re.sub(r'>.*', '', anno)
                anno = re.sub(r'\[.*', '', anno)
            else:
                continue
            if re.match(
                    r'Uncharacterized protein|uncharacterized protein|unknown|hypothetical protein|unnamed protein product|protein with unknown function|expressed protein|hCG\d{4,}|KIAA\d{4,}|predicted protein',
                    anno):
                continue
            if anno not in function_anno:
                function_anno.append(anno)
        output_dict[query_name] = function_anno
    return output_dict


def blast_plot(ax, hit, q_name, q_start, q_end, s_name, s_start, s_end):
    ax.set_xlim(q_start, q_end)
    ax.set_ylim(s_start, s_end)
    ax.set_title('blast plot')
    ax.set_xlabel(q_name)
    ax.set_ylabel(s_name)

    for hsp in hit.hsp:
        if not section((hsp.Hsp_query_from, hsp.Hsp_query_to), (q_start, q_end), just_judgement=True):
            continue

        if not section((hsp.Hsp_hit_from, hsp.Hsp_hit_to), (s_start, s_end), just_judgement=True):
            continue

        #     print((hsp.Hsp_query_from,hsp.Hsp_hit_from),(hsp.Hsp_query_to,hsp.Hsp_hit_to))

        ax.plot((hsp.Hsp_query_from, hsp.Hsp_query_to),
                (hsp.Hsp_hit_from, hsp.Hsp_hit_to), marker='.', c='k')


def BBH_call(bls_file1, bls_file2):
    """
    bidirectional best hit method
    """
    bls1_dict = outfmt6_read(bls_file1)
    bls2_dict = outfmt6_read(bls_file2)

    BBH_pair_list = []
    for q_id in bls1_dict:
        s_id = bls1_dict[q_id].hit[0].Hit_id
        if s_id in bls2_dict and q_id == bls2_dict[s_id].hit[0].Hit_id:
            BBH_pair_list.append((q_id, s_id))

    only_q1 = list(set([i for i in bls1_dict]) - set([i[0]
                                                      for i in BBH_pair_list]))
    only_q2 = list(set([i for i in bls2_dict]) - set([i[1]
                                                      for i in BBH_pair_list]))

    return BBH_pair_list, only_q1, only_q2


if __name__ == '__main__':
    """
    file = "C006N.fa.vs.ip"
    bls_dir = outfmt5_read(file)

    file2 = "/share/home/xuyuxing/Work/cuscuta/HGT/blast/bls/GCF_000001215.4/10.seq.bls"
    bls_dir2 = outfmt5_read(file2)
    """

    bls_file = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/WorkingDirectory/Blast0_1.txt.gz'
    db_name = '/lustre/home/xuyuxing/Work/Csp/orthofinder/protein_seq/Results_Apr10/WorkingDirectory/test2.db'

    blast_to_sqlite(db_name, bls_file, gzip_flag=True)

    query_tmp = next(sqlite_read(db_name))

    # plot
    import matplotlib.pyplot as plt

    hit_used = [hit for hit in query_tmp.hit if hit.Hit_id ==
                'G001N_fragment_2'][0]

    q_start = 1
    q_end = 35304
    s_start = 20277869
    s_end = 20321356

    fig, ax = plt.subplots(1, 1)

    blast_plot(ax, hit_used, 'MF163256.1', q_start - 3000, q_end + 3000, 'G001N_fragment_2', s_start - 3000,
               s_end + 3000)

    plt.show()


    ###
    bls_file = '/lustre/home/xuyuxing/Work/Orobanchaceae/Trans/clean_data/Oro/blat/Trinity.bls'
    query_file = '/lustre/home/xuyuxing/Database/Orobanche/annotation/trinity/trinity_output/Trinity.fasta'

    from lib.common.genome.seq_base import read_fasta_by_faidx

    seq_dict = read_fasta_by_faidx(query_file)
    len_dict = {i:seq_dict[i].len() for i in seq_dict}

    for query in outfmt6_read_big(bls_file):
        query.qLen = len_dict[query.qID]
        for hit in query.hit:
            print(query.qID,hit.Hit_id,hit_CIP(hit),hit_CALP(hit))


import os
import re
import pickle
from pyfaidx import Fasta
from lib.common.util import printer_list
from lib.common.util import logging_init, configure_parser
from lib.common.os import mkdir, copy_file, cmd_run, multiprocess_running
from src.xuyuxing.tools.seqtools import split_fasta
from lib.common.genome.seq_base import read_fasta
from argparse import Namespace
from api.xuyuxing.genome.annotation_shell_code import *
from lib.common.genome.genome_feature2 import read_gff_file
from api.xuyuxing.resource.dfam import build_TE_class_tree
from api.xuyuxing.resource.dfam import dfam_TE_classes_file as defaults_dfam_file
from lib.common.evolution.tree_operate import lookup_by_names


class Maker_P_Job(object):
    def __init__(self, genome_fasta_file, output_dir, config_file=None, tag='Genome', num_threads=56):
        # parse argument

        args = Namespace(genome=genome_fasta_file, output_dir=output_dir,
                         config_file=config_file, tag=tag, num_threads=num_threads)

        try:
            script_dir_path = os.path.split(os.path.realpath(__file__))[0]
            defaults_config_file = script_dir_path + \
                "/lib/config_file/Maker_p_defaults.ini"
        except:
            defaults_config_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/lib/config_file/Maker_p_defaults.ini"

        args_type = {
            # Parameter
            "num_threads": "int",
            "tag": "str",

            # Paths
            "genome": "str",
            "trna_file": "str",
            "crl_dir": "str",
            "tpase_dir": "str",
            "pe_dir": "str",
            "old_blastx": "str",
            "protein_db": "str"
        }

        args = configure_parser(
            args, defaults_config_file, args.config_file, args_type, None)
        self.args = args

        # running maker p
        self.maker_p_repeat_main()

    def step_1_3(sh_file, trna_file, crl_dir):
        with open(sh_file, 'w') as f:
            f.write(step_1_3_shell % (trna_file, crl_dir))

    def step_1_3_85(sh_file, trna_file, crl_dir):
        with open(sh_file, 'w') as f:
            f.write(step_1_3_85_shell % (trna_file, crl_dir))

    def step3_merge(work_dir, contig_list, out_dir, identity):
        """
        work_dir = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/repeat/LTR"
        list_file = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/repeat/LTR/list"
        out_dir = "/lustre/home/xuyuxing/Database/Phelipanche/annotation/maker_p/repeat/LTR/merge85"
        """

        identity = str(identity)

        contig_dir = {}
        num = 0
        for i in contig_list:
            contig_dir[i] = {}
            contig_dir[i]['seq-nr'] = str(num)
            contig_main_dir = work_dir + "/" + i
            if os.path.exists(contig_main_dir):
                CRL_Step3_file = contig_main_dir + "/CRL_Step3_Passed_Elements.fasta"
                result99_file = contig_main_dir + "/" + i + ".result%s" % identity
                outinner99_file = contig_main_dir + "/" + i + ".outinner%s" % identity

                if os.path.exists(CRL_Step3_file):
                    contig_dir[i]["CRL_Step3_file"] = CRL_Step3_file
                # else:
                #     print("%s miss CRL_Step3_file" % i)

                if os.path.exists(result99_file):
                    contig_dir[i]["result99_file"] = result99_file
                # else:
                #     print("%s miss result99_file" % i)

                if os.path.exists(outinner99_file):
                    contig_dir[i]["outinner99_file"] = outinner99_file
                # else:
                #     print("%s miss outinner99_file" % i)
            # else:
            #     print("%s miss main dir" % i)
            num = num + 1

        mkdir(out_dir, True)
        merge_CRL_Step3_file = out_dir + "/all.CRL_Step3_Passed_Elements.fasta"
        merge_result99 = out_dir + "/all.result%s" % identity
        merge_outinner99 = out_dir + "/all.outinner%s" % identity

        step3_f = open(merge_CRL_Step3_file, 'w')
        result99_f = open(merge_result99, 'w')
        outinner99_f = open(merge_outinner99, 'w')

        for contig in contig_dir:
            if "CRL_Step3_file" in contig_dir[contig]:
                CRL_Step3_file = contig_dir[contig]["CRL_Step3_file"]
                seqdict, seqname_list = read_fasta(CRL_Step3_file)
                for i in seqname_list:
                    new_seq_name = re.sub(
                        'dbseq-nr_0', 'dbseq-nr_%s' % contig_dir[contig]['seq-nr'], i)
                    step3_f.write('>%s\n%s\n' %
                                  (new_seq_name, seqdict[i].seq.lower()))

            if "result99_file" in contig_dir[contig]:
                result99_file = contig_dir[contig]["result99_file"]
                with open(result99_file, 'r') as f:
                    for each_line in f:
                        if re.match(r'^#', each_line):
                            continue
                        each_line = re.sub('\n', '', each_line)
                        info = each_line.split()
                        info[-1] = str(contig_dir[contig]['seq-nr'])
                        result99_f.write(printer_list(info, sep="  ") + "\n")

            if "outinner99_file" in contig_dir[contig]:
                outinner99_file = contig_dir[contig]["outinner99_file"]
                seqdict, seqname_list = read_fasta(
                    outinner99_file, full_name=True, upper=False)
                for i in seqname_list:
                    new_seq_name = re.sub(
                        'dbseq-nr 0', 'dbseq-nr %s' % contig_dir[contig]['seq-nr'], i)
                    seqdict[i].wrap(60)
                    outinner99_f.write('>%s\n%s' %
                                       (new_seq_name, seqdict[i].seq.lower()))

        step3_f.close()
        result99_f.close()
        outinner99_f.close()

    def step_4_5(sh_file, genome, trna_file, crl_dir, mite_file, tpase_dir):
        with open(sh_file, 'w') as f:
            f.write(step_4_5_shell %
                    (genome, trna_file, crl_dir, mite_file, tpase_dir))

    def step_4_5_85(sh_file, genome, trna_file, crl_dir, mite_file, tpase_dir):
        with open(sh_file, 'w') as f:
            f.write(step_4_5_85_shell %
                    (genome, trna_file, crl_dir, mite_file, tpase_dir))

    def maker_p_repeat_main(self):
        args = self.args

        mkdir(args.output_dir, True)
        log_file = args.output_dir + "/log.txt"
        args.genome = os.path.abspath(args.genome)
        args.output_dir = os.path.abspath(args.output_dir)

        logger = logging_init("maker_p repeat build", log_file)

        args_info_string = "Argument list:\n"
        attrs = vars(args)

        for item in attrs.items():
            args_info_string = args_info_string + ("%s: %s\n" % item)

        logger.info(args_info_string)

        logger.info("Step1: MITE find")

        MITE_dir = args.output_dir + "/MITE"
        MITE_pass_flag_file = MITE_dir + "/MITE.ok"

        if not os.path.exists(MITE_pass_flag_file):
            logger.info("   MITE find, begin")

            bash_file = MITE_dir + "/MITE.sh"
            mkdir(MITE_dir, True)

            with open(bash_file, 'w') as f:
                f.write(MITE_shell % (args.genome, MITE_dir, args.mite_hunter_manager,
                                      args.tag, args.num_threads, args.num_threads))

            code, output, error = cmd_run("bash %s" % bash_file, cwd=MITE_dir)

            stdout_file = MITE_dir + "/MITE.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = MITE_dir + "/MITE.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            MITE_file = MITE_dir + "/MITE.lib"

            if code and os.path.getsize(MITE_file) != 0:
                MITE_pass_flag_file = MITE_dir + "/MITE.ok"
                cmd_run("touch %s" % MITE_pass_flag_file, cwd=MITE_dir)
                logger.info("   MITE find, finished")
            else:
                logger.info("   MITE find, failed, please check")
                raise ValueError("MITE failed, please check")

        else:
            logger.info("   MITE find, already finished, skip step")
            MITE_file = MITE_dir + "/MITE.lib"

        logger.info("Step2: genome split")
        split_genome_dir = args.output_dir + "/split_genome"
        mkdir(split_genome_dir, True)

        genome_ref = Fasta(args.genome)
        contig_list = list(genome_ref.keys())
        split_ok = True
        for i in contig_list:
            contig_file = split_genome_dir + "/" + i + ".fa"
            if not (os.path.exists(contig_file) and os.path.getsize(contig_file) != 0):
                split_ok = False

        if not split_ok:
            mkdir(split_genome_dir, True)
            split_fasta(args.genome, split_genome_dir, None, True)

        logger.info("Step3: LTRs find")
        ltr_dir = args.output_dir + "/LTR"
        mkdir(ltr_dir, True)

        logger.info("Step3.1: LTRs that are 99% or more in similarity")
        ltr_99_dir = ltr_dir + "/99"
        ltr_99_log = ltr_99_dir + "/log.txt"
        mkdir(ltr_99_dir, True)
        step_1_3_sh = ltr_99_dir + "/step_1_3.sh"
        step_1_3(step_1_3_sh, args.trna_file, args.crl_dir)
        tmp_out_pickle = ltr_99_dir + "/shell_para_out.pyb"

        logger.info("   raw step 2.1.1 - 2.1.3")

        step_1_3_flag = False
        if os.path.exists(tmp_out_pickle):
            # TEMP = open(tmp_out_pickle, 'rb')
            # tmp_out = pickle.load(TEMP)
            # TEMP.close()

            crl_step_file_num = {
                'step1': [0, 0],
                'step2': [0, 0],
                'step3': [0, 0],
                'dgt': [0, 0],
                'result99': [0, 0]
            }

            for contig in contig_list:
                crl_step_file = {
                    'step1': "%s/%s/CRL_Step1_Passed_Elements.txt" % (ltr_99_dir, contig),
                    'step2': "%s/%s/fasta_files/CRL_Step2_Passed_Elements.fasta" % (ltr_99_dir, contig),
                    'step3': "%s/%s/CRL_Step3_Passed_Elements.fasta" % (ltr_99_dir, contig),
                    'dgt': "%s/%s/%s.gff99.dgt" % (ltr_99_dir, contig, contig),
                    'result99': "%s/%s/%s.result99" % (ltr_99_dir, contig, contig),
                }

                for step in list(crl_step_file.keys()):
                    if os.path.exists(crl_step_file[step]):
                        crl_step_file_num[step][0] += 1
                        if os.path.getsize(crl_step_file[step]) != 0:
                            crl_step_file_num[step][1] += 1

            if crl_step_file_num['step1'][0] == crl_step_file_num['step2'][0] == crl_step_file_num['dgt'][0] == \
                    crl_step_file_num['result99'][1] == len(contig_list):
                if crl_step_file_num['step1'][1] != 0 and crl_step_file_num['step2'][1] != 0 and crl_step_file_num['step3'][
                        1] != 0:
                    step_1_3_flag = True
                    logger.info("   raw step 1-3, already finished, skip step")

        if not step_1_3_flag:

            args_list = []
            for contig in contig_list:
                contig_fasta_file = os.path.abspath(
                    split_genome_dir + "/" + contig + ".fa")
                cmd_string = "bash %s %s %s" % (
                    step_1_3_sh, contig_fasta_file, contig)
                args_list.append((cmd_string, ltr_99_dir, 1, True, None))

            logger.info("   raw step 1-3, begin")

            tmp_out = multiprocess_running(
                cmd_run, args_list, args.num_threads, log_file=ltr_99_log)

            OUT = open(tmp_out_pickle, 'wb')
            pickle.dump(tmp_out, OUT)
            OUT.close()

            logger.info("   raw step 1-3, finished")

        logger.info(
            "   merge CRL_Step3_Passed_Elements.fasta，$TAG.result99，$TAG.outinner99")
        merge_dir = ltr_99_dir + "/merge"
        mkdir(merge_dir, True)
        step3_merge(ltr_99_dir, contig_list, merge_dir, 99)

        logger.info("   raw step 2.1.4 - 2.1.5")

        LTR99_lib_file = merge_dir + "/LTR99.lib"
        if os.path.exists(LTR99_lib_file):
            logger.info("   raw step 4-5, already finished, skip step")
        else:
            logger.info("   raw step 4-5, begin")
            step_4_5_sh = merge_dir + "/step_4_5.sh"
            step_4_5(step_4_5_sh, args.genome, args.trna_file,
                     args.crl_dir, MITE_file, args.tpase_dir)
            code, output, error = cmd_run(
                "bash %s" % step_4_5_sh, cwd=merge_dir)

            stdout_file = merge_dir + "/step_4_5.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = merge_dir + "/step_4_5.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   raw step 4-5, finished")

        logger.info("Step3.2: LTRs that are 85% or more in similarity")
        ltr_85_dir = ltr_dir + "/85"
        ltr_85_log = ltr_85_dir + "/log.txt"
        mkdir(ltr_85_dir, True)
        step_1_3_85_sh = ltr_85_dir + "/step_1_3.sh"
        step_1_3_85(step_1_3_85_sh, args.trna_file, args.crl_dir)
        tmp_out_pickle = ltr_85_dir + "/shell_para_out.pyb"

        logger.info("   raw step 2.1.1 - 2.1.3")

        step_1_3_85_flag = False
        if os.path.exists(tmp_out_pickle):
            # TEMP = open(tmp_out_pickle, 'rb')
            # tmp_out = pickle.load(TEMP)
            # TEMP.close()

            crl_step_file_num = {
                'step1': [0, 0],
                'step2': [0, 0],
                'step3': [0, 0],
                'dgt': [0, 0],
                'result85': [0, 0]
            }

            for contig in contig_list:
                crl_step_file = {
                    'step1': "%s/%s/CRL_Step1_Passed_Elements.txt" % (ltr_85_dir, contig),
                    'step2': "%s/%s/fasta_files/CRL_Step2_Passed_Elements.fasta" % (ltr_85_dir, contig),
                    'step3': "%s/%s/CRL_Step3_Passed_Elements.fasta" % (ltr_85_dir, contig),
                    'dgt': "%s/%s/%s.gff85.dgt" % (ltr_85_dir, contig, contig),
                    'result85': "%s/%s/%s.result85" % (ltr_85_dir, contig, contig),
                }

                for step in list(crl_step_file.keys()):
                    if os.path.exists(crl_step_file[step]):
                        crl_step_file_num[step][0] += 1
                        if os.path.getsize(crl_step_file[step]) != 0:
                            crl_step_file_num[step][1] += 1

            if crl_step_file_num['step1'][0] == crl_step_file_num['step2'][0] == crl_step_file_num['dgt'][0] == \
                    crl_step_file_num['result85'][1] == len(contig_list):
                if crl_step_file_num['step1'][1] != 0 and crl_step_file_num['step2'][1] != 0 and crl_step_file_num['step3'][
                        1] != 0:
                    step_1_3_85_flag = True
                    logger.info("   raw step 1-3, already finished, skip step")

        if not step_1_3_85_flag:

            args_list = []
            for contig in contig_list:
                contig_fasta_file = os.path.abspath(
                    split_genome_dir + "/" + contig + ".fa")
                cmd_string = "bash %s %s %s" % (
                    step_1_3_85_sh, contig_fasta_file, contig)
                args_list.append((cmd_string, ltr_85_dir, 1, True, None))

            logger.info("   raw step 1-3, begin")

            tmp_out = multiprocess_running(
                cmd_run, args_list, args.num_threads, log_file=ltr_85_log)

            OUT = open(tmp_out_pickle, 'wb')
            pickle.dump(tmp_out, OUT)
            OUT.close()

            logger.info("   raw step 1-3, finished")

        logger.info(
            "   merge CRL_Step3_Passed_Elements.fasta，$TAG.result85，$TAG.outinner85")
        merge_dir = ltr_85_dir + "/merge"
        mkdir(merge_dir, True)
        step3_merge(ltr_85_dir, contig_list, merge_dir, 85)

        logger.info("   raw step 2.1.4 - 2.1.5")

        LTR85_lib_file = merge_dir + "/LTR85.lib"
        if os.path.exists(LTR85_lib_file):
            logger.info("   raw step 4-5, already finished, skip step")
        else:
            logger.info("   raw step 4-5, begin")
            step_4_5_85_sh = merge_dir + "/step_4_5.sh"
            step_4_5_85(step_4_5_85_sh, args.genome, args.trna_file,
                        args.crl_dir, MITE_file, args.tpase_dir)
            code, output, error = cmd_run(
                "bash %s" % step_4_5_85_sh, cwd=merge_dir)

            stdout_file = merge_dir + "/step_4_5.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = merge_dir + "/step_4_5.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   raw step 4-5, finished")

        logger.info("Step3.3: merge 85 and 99 lib")

        allLTR_file = merge_dir + "/allLTR.lib"
        if os.path.exists(allLTR_file):
            logger.info("   already finished, skip step")
        else:
            logger.info("   begin")
            copy_file(LTR99_lib_file, merge_dir)

            bash_file = merge_dir + "/merge85_99.sh"

            with open(bash_file, 'w') as f:
                f.write(merge85_99_shell % args.crl_dir)

            code, output, error = cmd_run("bash %s" % bash_file, cwd=merge_dir)

            stdout_file = merge_dir + "/merge85_99.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = merge_dir + "/merge85_99.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            allLTR_file = merge_dir + "/allLTR.lib"

            logger.info("   finished")

        logger.info("Step4: Collecting repetitive sequences by RepeatModeler")

        repeat_dir = args.output_dir + "/Repeat"
        mkdir(repeat_dir, True)
        copy_file(allLTR_file, repeat_dir)
        copy_file(MITE_file, repeat_dir)

        cmd_run("cat allLTR.lib MITE.lib > allMITE_LTR.lib", cwd=repeat_dir)
        alllib_file = os.path.abspath(repeat_dir + "/allMITE_LTR.lib")

        logger.info("   RepeatMasker")

        tmp_out_pickle = repeat_dir + "/RepeatMasker.pyb"

        if os.path.exists(tmp_out_pickle):
            logger.info("   RepeatMasker, finished, skip step")
        else:
            bash_file = repeat_dir + "/RepeatMasker.sh"
            with open(bash_file, 'w') as f:
                f.write(RepeatMasker_shell % alllib_file)

            args_list = []
            for contig in contig_list:
                contig_fasta_file = os.path.abspath(
                    split_genome_dir + "/" + contig + ".fa")
                cmd_string = "bash %s %s %s" % (
                    bash_file, contig_fasta_file, contig)
                args_list.append((cmd_string, repeat_dir, 1, True, None))

            logger.info("   RepeatMasker, begin")

            tmp_out = multiprocess_running(
                cmd_run, args_list, args.num_threads, log_file=ltr_99_log)

            OUT = open(tmp_out_pickle, 'wb')
            pickle.dump(tmp_out, OUT)
            OUT.close()

            logger.info("   RepeatMasker, finished")

        logger.info("   RepeatModeler")

        ModelerID_lib_file = repeat_dir + "/ModelerID.lib"
        ModelerUnknown_lib_file = repeat_dir + "/ModelerUnknown.lib"

        if os.path.exists(ModelerID_lib_file):
            logger.info("   RepeatModeler, finished, skip step")
        else:
            bash_file = repeat_dir + "/RepeatModeler.sh"
            with open(bash_file, 'w') as f:
                f.write(RepeatModeler_shell % (args.crl_dir, args.tpase_dir))

            logger.info("   RepeatModeler, begin")
            code, output, error = cmd_run(
                "bash %s" % bash_file, cwd=repeat_dir)

            stdout_file = repeat_dir + "/RepeatModeler.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = repeat_dir + "/RepeatModeler.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   RepeatModeler, finished")

        logger.info("Step5: Exclusion of gene fragments")

        no_gene_dir = args.output_dir + "/NoGeneFrag"
        mkdir(no_gene_dir, True)
        allRepeats_lib_file = no_gene_dir + "/allRepeats.lib"

        if os.path.exists(allRepeats_lib_file):
            logger.info("   Step5, finished, skip step")
        else:
            copy_file(allLTR_file, no_gene_dir)
            copy_file(MITE_file, no_gene_dir)
            copy_file(ModelerUnknown_lib_file, no_gene_dir)
            copy_file(ModelerID_lib_file, no_gene_dir)

            logger.info("   Blastx, round 1")
            code, output, error = cmd_run(
                "%s -num_threads 56 -query ModelerUnknown.lib -db %s -evalue 1e-10 -num_descriptions 10 -out ModelerUnknown.lib_blast_results.txt" % (
                    args.old_blastx,
                    args.protein_db
                ), cwd=no_gene_dir)

            stdout_file = no_gene_dir + "/blastx.r1.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = no_gene_dir + "/blastx.r1.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   Blastx, round 2")
            code, output, error = cmd_run(
                "%s -num_threads 56 -query MITE.lib -db %s -evalue 1e-10 -num_descriptions 10 -out MITE.lib_blast_results.txt" % (
                    args.old_blastx,
                    args.protein_db
                ), cwd=no_gene_dir)

            stdout_file = no_gene_dir + "/blastx.r2.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = no_gene_dir + "/blastx.r2.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   Blastx, round 3")
            code, output, error = cmd_run(
                "%s -num_threads 56 -query allLTR.lib -db %s -evalue 1e-10 -num_descriptions 10 -out allLTR.lib_blast_results.txt" % (
                    args.old_blastx,
                    args.protein_db
                ), cwd=no_gene_dir)

            stdout_file = no_gene_dir + "/blastx.r3.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = no_gene_dir + "/blastx.r3.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   Blastx, round 4")
            code, output, error = cmd_run(
                "%s -num_threads 56 -query ModelerID.lib -db %s -evalue 1e-10 -num_descriptions 10 -out ModelerID.lib_blast_results.txt" % (
                    args.old_blastx,
                    args.protein_db
                ), cwd=no_gene_dir)

            stdout_file = no_gene_dir + "/blastx.r4.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = no_gene_dir + "/blastx.r4.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

            logger.info("   get final lib")
            bash_file = no_gene_dir + "/get_final_lib.sh"
            with open(bash_file, 'w') as f:
                f.write(get_final_lib_shell % args.pe_dir)

            code, output, error = cmd_run(
                "bash %s" % bash_file, cwd=no_gene_dir)

            stdout_file = no_gene_dir + "/get_final_lib.sh.o"
            with open(stdout_file, 'w') as f:
                f.write(output)

            stderr_file = no_gene_dir + "/get_final_lib.sh.e"
            with open(stderr_file, 'w') as f:
                f.write(error)

        copy_file(allRepeats_lib_file, args.output_dir)
        logger.info("Everything is finished")

    def results_gather(self):
        work_dir = self.args.output_dir

        genome_ref = Fasta(self.args.genome)
        contig_list = list(genome_ref.keys())

        self.raw_results = {
            "MITE": work_dir + "/MITE/MITE.lib",
            "ltrharvest_99_result": work_dir + "/99/merge/all.result99",
            "MITE": work_dir + "/MITE/MITE.lib",
            "MITE": work_dir + "/MITE/MITE.lib",
            "MITE": work_dir + "/MITE/MITE.lib",
            "MITE": work_dir + "/MITE/MITE.lib",
        }


class LTR_retriever_Job(object):
    pass


def parse_maker_repeat_results(results_gff_file, te_tree, add_map_dict=None, query_lineage_dict=None, MITE_seq_id_list=None, de_novo_LTR_id_list=None):
    results_gff_info_dict = read_gff_file(results_gff_file)

    def if_coding(gf):
        if len(gf.sub_features) > 0:
            if gf.sub_features[0].type == 'mRNA':
                return True
        return False

    def if_snoRNA(gf):
        if len(gf.sub_features) > 0:
            if gf.sub_features[0].type == 'snoRNA':
                return True
        return False

    def if_tRNA(gf):
        if len(gf.sub_features) > 0:
            if gf.sub_features[0].type == 'tRNA':
                return True
        return False

    def if_repeat(gf):
        if gf.qualifiers['source'][0] == 'repeatmasker' or gf.qualifiers['source'][0] == 'repeatrunner':
            return True
        return False

    # build tc tree to save 

    tc_dict = {}
    for tc in te_tree.find_clades(order='preorder'):
        if hasattr(tc, 'lineage'):
            tc_dict[tc.lineage] = tc

   # reads maker output

    coding_gene_dict = {}
    snoRNA_gene_dict = {}
    tRNA_gene_dict = {}
    repeat_dict = {}

    for top_type in results_gff_info_dict:
        top_dict = results_gff_info_dict[top_type]

        for gf_id in top_dict:
            gf = top_dict[gf_id]

            # coding gene
            if if_coding(gf):
                coding_gene_dict[gf_id] = gf

            # snoRNA gene
            if if_snoRNA(gf):
                snoRNA_gene_dict[gf_id] = gf

            # tRNA gene
            if if_tRNA(gf):
                tRNA_gene_dict[gf_id] = gf

            # repeat
            if if_repeat(gf):
                repeat_dict[gf_id] = gf

    # repeatmasker results
    repeatmasker_tc_map = {}
    for i in tc_dict:
        tc = tc_dict[i]
        repeatmasker_equiv = tc.db_id_map['repeatmasker']
        if not repeatmasker_equiv == ("", ""):
            repeatmasker_tc_map[repeatmasker_equiv] = i

    if add_map_dict:
        for i in add_map_dict:
            repeatmasker_tc_map[i] = add_map_dict[i]

    genus_results_dict = {}
    MITE_dict = {}
    repeatmasker_unknown_gf_dict = {}
    failed_gf = []
    all_species_list = []
    all_genus_list = []
    for gf_id in repeat_dict:
        gf = repeat_dict[gf_id]

        if gf.qualifiers['source'][0] == 'repeatmasker':
            gf_Name = gf.qualifiers['Name'][0]
            repeat_name_dict = {}
            for i in gf_Name.split("|"):
                repeat_name_dict[i.split(":")[0]] = i.split(":")[1]

            try:
                genus = repeat_name_dict['genus']
                species = repeat_name_dict['species']

                all_species_list.append(species)
                all_genus_list.append(genus)

                # MITE
                if MITE_seq_id_list and species in set(MITE_seq_id_list):
                    if species not in MITE_dict:
                        MITE_dict[species] = []
                    MITE_dict[species].append(gf)    
                    continue

                if genus == 'Unknown' or genus == 'Unspecified':
                    if species not in repeatmasker_unknown_gf_dict:
                        repeatmasker_unknown_gf_dict[species] = []
                    repeatmasker_unknown_gf_dict[species].append(gf)    
                    continue

                if genus not in genus_results_dict:
                    genus_results_dict[genus] = {}

                if species not in genus_results_dict[genus]:
                    genus_results_dict[genus][species] = []

                genus_results_dict[genus][species].append(gf)

            except:
                failed_gf.append(gf)

    for genus_word in genus_results_dict:
        repeatmasker_k1 = genus_word.split("/")[0].split("?")[0]
        if "/" in genus_word:
            repeatmasker_k2 = genus_word.split("/")[-1].split("?")[0]
        else:
            repeatmasker_k2 = ""

        repeatmasker_equiv = (repeatmasker_k1, repeatmasker_k2)

        if repeatmasker_equiv in repeatmasker_tc_map:
            tc_lineage = repeatmasker_tc_map[repeatmasker_equiv]
            tc = tc_dict[tc_lineage]

            if hasattr(tc, 'repeat_case_dict'):
                for i in genus_results_dict[genus_word]:
                    tc.repeat_case_dict[i] = genus_results_dict[genus_word][i]
            else:
                tc.repeat_case_dict = genus_results_dict[genus_word]

        else:
            raise ValueError("Unknown repeatmasker_equiv: %s %s" %
                             (repeatmasker_equiv[0], repeatmasker_equiv[1]))

    # add query_lineage_dict info
    used_species = []
    for species in repeatmasker_unknown_gf_dict:
        if species in query_lineage_dict:
            anno_lineage = query_lineage_dict[species]
            
            tc = tc_dict[anno_lineage]

            if hasattr(tc, 'repeat_case_dict'):
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]
            else:
                tc.repeat_case_dict = {}
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]

            used_species.append(species)

    # add unknown de novo LTR
    ltr_lineage = ('Interspersed_Repeat', 'Transposable_Element', 'Retrotransposed_Element', 'Retrotransposon', 'Long_Terminal_Repeat_Element')
    tc = tc_dict[ltr_lineage]
    for species in repeatmasker_unknown_gf_dict:
        if species in set(de_novo_LTR_id_list) and species not in set(used_species):
            if hasattr(tc, 'repeat_case_dict'):
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]
            else:
                tc.repeat_case_dict = {}
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]
            used_species.append(species)

    # add other unknown to tc
    ltr_lineage = ('Interspersed_Repeat', 'Unknown')
    tc = tc_dict[ltr_lineage]
    for species in repeatmasker_unknown_gf_dict:
        if species not in set(used_species):
            if hasattr(tc, 'repeat_case_dict'):
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]
            else:
                tc.repeat_case_dict = {}
                tc.repeat_case_dict[species] = repeatmasker_unknown_gf_dict[species]
            # used_species.append(species)    


    # repeatrunner results
    # pass

    # tRNA
    tRNA_family_dict = {}
    for tRNA_id in tRNA_gene_dict:
        match = re.findall("trnascan-.*-noncoding-(\S+)_(\S+)-gene-.*", tRNA_id)
        if len(match) > 0:
            aa, code = match[0]
            if not (aa, code) in tRNA_family_dict:
                tRNA_family_dict[(aa, code)] = []
            tRNA_family_dict[(aa, code)].append(tRNA_gene_dict[tRNA_id])
        else:
            raise ValueError("Unknown tRNA id: %s" % tRNA_id)
    
    tRNA_lineage = ('Interspersed_Repeat', 'Pseudogene', 'RNA', 'tRNA')
    tc = tc_dict[tRNA_lineage]
    tc.repeat_case_dict = tRNA_family_dict

    return te_tree, MITE_dict


if __name__ == '__main__':
    from api.xuyuxing.resource.dfam import get_seq_anntation, build_TE_class_tree
    from lib.common.fileIO import read_list_file

    add_map_dict = {
        ('Satellite', 'subtelo'): ('Tandem_Repeat', 'Satellite', 'Subtelomeric'),
        ('Satellite', 'acro'): ('Tandem_Repeat', 'Satellite', 'Acromeric'),
        ('Satellite', 'centr'): ('Tandem_Repeat', 'Satellite', 'Centromeric'),
        ('Unspecified', ''): ('Other',),
    }


    results_gff_file = '/lustre/home/xuyuxing/Work/Gel/GenomeStat/Gel.genome.v2.0.results.gff'
    dfamtblout = "/lustre/home/xuyuxing/Work/Gel/GenomeStat/allRepeats.vs.dfam/nhmmscan.dfamtblout"
    dfam_db = "/lustre/home/xuyuxing/Database/Dfam/Dfam.info.db"
    MITE_id_file = "/lustre/home/xuyuxing/Work/Gel/GenomeStat/MITE.id"
    denovo_LTR_file = "/lustre/home/xuyuxing/Work/Gel/GenomeStat/denovo_LTR.id"

    te_tree = build_TE_class_tree()

    query_lineage_dict_raw = get_seq_anntation(dfamtblout, dfam_db, e_value_thr=1e-5, cover=0.6)
    query_lineage_dict = {}
    for i in query_lineage_dict_raw:
        query_lineage_dict[i.split("#")[0]] = query_lineage_dict_raw[i]

    MITE_seq_id_list = read_list_file(MITE_id_file)
    de_novo_LTR_id_list = read_list_file(denovo_LTR_file)

    te_tree, MITE_dict = parse_maker_repeat_results(
        results_gff_file, te_tree, add_map_dict=add_map_dict, query_lineage_dict=query_lineage_dict, MITE_seq_id_list=MITE_seq_id_list, de_novo_LTR_id_list=de_novo_LTR_id_list)

    #

    from lib.common.math.interval import merge_intervals, sum_interval_length


    def gf_sum_length(gf_list):
        chr_dict = {}
        for gf in gf_list:
            if not gf.chr_id in chr_dict:
                chr_dict[gf.chr_id] = []
            chr_dict[gf.chr_id].append(gf.range)
        
        sum_length = 0
        for i in chr_dict:
            merged_range = merge_intervals(chr_dict[i])
            length_now = sum_interval_length(merged_range)
            sum_length += length_now
        
        return sum_length
        
    tc_dict = {}
    for tc in te_tree.find_clades(order='preorder'):
        if hasattr(tc, 'lineage'):
            lineage = tc.lineage
            tc_dict[lineage] = tc
            if hasattr(tc, 'repeat_case_dict'):
                sum_gf_list = []
                for i in tc.repeat_case_dict:
                    gf_list = tc.repeat_case_dict[i]
                    sum_gf_list.extend(gf_list)
                print(lineage, gf_sum_length(sum_gf_list))                